@@ -13,6 +13,13 @@
 /* Defined only when compiing on some brand of Solaris system. */
 #undef Solaris
 
+/* Defined when your system has a define or typedef for sig_t. */
+#undef HAVE_TYPE_SIG_T
+
+
+/* Defined when your system has a define or typedef for time_t. */
+#undef HAVE_TYPE_TIME_T
+
 @BOTTOM@
 #if defined (Solaris)
 #  include <arch/solaris/prototypes.h>
