@@ -207,7 +207,8 @@ rigidly along with this one."
      "ul" "ol" "dl" "table" "tr" "td" "center" "a" "form" "select" "caption"
      "applet" "subtable" "in-package" "with-local-package"
      "example" "question" "nested-items-list" "nested-item" "slide" "bullet"
-     "menu" "layout::page" "layout::left-border" "layout::right-border"
+     "menu" "system-menu"
+     "layout::page" "layout::left-border" "layout::right-border"
      "frameset" "msql::with-open-database" "odbc::with-open-database"
      "dbm::with-open-database" "def-pagelist")
   "List of tag starts which are complex.")
