@@ -28,50 +28,32 @@
    Santa Barbara, CA
    93101  */
 
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
+#include "language.h"
+#include "http.h"
+#include "globals.h"
+#include "logging.h"
+#if defined (HAVE_GETPWNAM)
+#  include <pwd.h>
+#endif
+
+#if defined (__WINNT__)
+#include <termios.h>
+#endif
+
+#if defined (BUILDING_WITH_FAST_CGI)
+#  include "./fcgi/include/fcgi_stdio.h"
+#endif
+
+#if !defined (BUILDING_WITH_FAST_CGI)
+#  if defined (_FCGI_STDIO)
+#    define BUILDING_WITH_FAST_CGI 1
+#  endif
 #endif
 
 #if !defined (MHTML_SYSTEM_TYPE)
 #  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
 #endif
 
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#if defined (sgi)
-#include <bstring.h>
-#endif
-#include <errno.h>
-#include <sys/types.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <fcntl.h>
-#  endif
-#endif /* !Solaris */
-#include <sys/ioctl.h>
-#include <sys/stat.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "pages.h"
-#include "symbols.h"
-#include "parser.h"
-#include "session.h"
-#include "streamfuncs.h"
-#include "http.h"
-#include "globals.h"
-#include "logging.h"
-
-#if defined (HAVE_GETPWNAM)
-#  include <pwd.h>
-#endif
-
 /* The name of this program, as taken from argv[0]. */
 static char *rawprogname = (char *)NULL;
 
@@ -90,13 +72,19 @@ static char *engine_config_path = (char *)NULL;
 /* Non-zero means this engine is named "nph-" something. */
 static int nph = 0;
 
+#if !defined (MHTML_VERSION_STRING)
+static char *mhtml_version_string = "";
+#else
+static char *mhtml_version_string = MHTML_VERSION_STRING;
+#endif
+
 /* Forward declarations. */
 static void parse_program_args (int argc, char *argv[]);
 static void initialize_engine (void);
 static void fatal (char *format, ...);
 
-#if defined (CHECK_TIMESTAMP)
-static void check_timestamp (char *filename);
+#if defined (CHECK_ACTIVATION)
+extern int check_activation_key (char *key);
 #endif
 
 static void process_request (void);
@@ -110,6 +98,9 @@ int
 main (int argc, char *argv[])
 {
   char *temp;
+#if defined (BUILDING_WITH_FAST_CGI)
+  int times_called = -1;
+#endif
 
   if ((argc > 0) && (argv[0] != (char *)NULL) && (argv[0][0] == '/'))
     rawprogname = strdup (argv[0]);
@@ -125,36 +116,47 @@ main (int argc, char *argv[])
   else
     progname = strdup (rawprogname);
 
+#if !defined (BUILDING_WITH_FAST_CGI)
   nph = (strncasecmp (progname, "nph-", 4) == 0);
-  parse_program_args (argc, argv);
-
-#if defined (CHECK_TIMESTAMP)
-  check_timestamp (argv[0]);
 #endif
+
+  parse_program_args (argc, argv);
   initialize_engine ();
+
+#if defined (BUILDING_WITH_FAST_CGI)
+  while (FCGI_Accept () >= 0)
+    {
+      pid_t child = (pid_t)0;
+
+      /* We got called again. */
+      times_called++;
+
+      /* So start a child to handle the request. */
+      child = fork ();
+
+      if (child != (pid_t) 0)
+	{
+	  int status;
+	  /* In the Parent. */
+	  wait (&status);
+	}
+      else
+	{
+	  char number[20];
+	  sprintf (number, "%d", times_called);
+	  pagefunc_set_variable ("fcgi::times-called", number);
+	  process_request ();
+	  fflush (stdout);
+	  _exit (0);
+	}
+    }
+#else
   process_request ();
+#endif /* !BUILDING_WITH_FAST_CGI */
+
   exit (0);
 }
 
-#if defined (CHECK_TIMESTAMP)
-#include "../libtimestamp/timestamp.h"
-
-static void
-check_timestamp (char *filename)
-{
-  if (check_stamp_expired (filename))
-    {
-      fprintf (stderr, "Sorry, but your free trial has ended.\n");
-      fprintf (stderr, "You may obtain another copy of the Engine from\n");
-      fprintf (stderr, "\n\thttp://www.metahtml.com\n\n");
-      fprintf (stderr, "We hope that you have enjoyed the use of our\n");
-      fprintf (stderr, "engine, and will continue to watch our site for\n");
-      fprintf (stderr, "new additions to our product suite.\n");
-      exit (2);
-    }
-}
-#endif /* CHECK_TIMESTAMP */
-
 /* How to initialize the Engine. */
 static void
 initialize_engine (void)
@@ -163,14 +165,22 @@ initialize_engine (void)
   PAGE *page;
   char *include_prefix = (char *)NULL;
 
-  pagefunc_set_variable ("mhttpd::copyright-string",metahtml_copyright_string);
-  pagefunc_set_variable ("mhtml::system-type", MHTML_SYSTEM_TYPE);
-
   /* Set up the input and output file descriptors. */
   mhtml_stdout_fileno = fileno (stdout);
   mhtml_stdin_fileno = fileno (stdin);
   mhtml_stderr_fileno = fileno (stderr);
 
+#if defined (__WINNT__)
+  setmode (mhtml_stdout_fileno, O_BINARY);
+  setmode (mhtml_stdin_fileno, O_BINARY);
+#endif /* ! __WINNT__ */
+
+  pagefunc_set_variable ("mhtml::version", mhtml_version_string);
+  pagefunc_set_variable ("mhttpd::copyright-string",metahtml_copyright_string);
+  pagefunc_set_variable ("mhtml::system-type", MHTML_SYSTEM_TYPE);
+
+  bootstrap_metahtml (0);
+
   /* Quickly make a package containing the minimum mime-types. */
   pagefunc_set_variable ("mime-type::.mhtml", "metahtml/interpreted");
   pagefunc_set_variable ("mime-type::.jpeg", "image/jpeg");
@@ -218,7 +228,11 @@ index.html\nhome.html");
     {
       engine_config_path = (char *)getenv ("METAHTML_ENGINE_CONFIG_FILE");
       if (engine_config_path == (char *)NULL)
+#if defined (__WINNT__)
+	engine_config_path = "/METAHTML/bin/engine.conf";
+#else
 	engine_config_path = "/www/bin/engine.conf";
+#endif /* __WINNT__ */
     }
 
   page = page_read_template (engine_config_path);
@@ -234,6 +248,15 @@ index.html\nhome.html");
   if (page)
     page_free_page (page);
 
+#if defined (CHECK_ACTIVATION)
+  {
+    char *key = pagefunc_get_variable ("mhtml::activation-key");
+    if (check_activation_key (key) == 0)
+      exit (1);
+    symbol_free (symbol_remove ("mhtml::activation-key"));
+  }
+#endif /* CHECK_ACTIVATION */
+
   /* If the user didn't set mhtml::require-directories[], give a reasonable
      value here. */
   temp = pagefunc_get_variable ("mhtml::require-directories");
@@ -295,7 +318,7 @@ index.html\nhome.html");
       bprintf (page, "<set-var mhtml::exec-path =");
       bprintf (page, "<get-var mhtml::server-root>/bin:/bin:");
       bprintf (page, "<get-var mhtml::document-root>/cgi-bin:/usr/bin:");
-      bprintf (page, "/usr/local/bin:/usr/ucb");
+      bprintf (page, "/usr/local/bin:/usr/ucb:/www/bin:/opt/metahtml/bin");
       bprintf (page, ">");
       page_process_page (page);
 
@@ -440,9 +463,18 @@ parse_program_args (int argc, char *argv[])
 	{
 	  engine_config_path = strdup (argv[arg_index++]);
 	}
+      else if ((strcasecmp (arg, "--key") == 0) ||
+	       (strcasecmp (arg, "-key") == 0))
+	{
+	  pagefunc_set_variable ("mhtml::activation-key", argv[arg_index++]);
+	}
       else
 	{
-	  if ((char *)NULL == (char *)getenv ("PATH_INFO"))
+	  if ((char *)NULL == (char *)getenv ("PATH_INFO")
+#if defined (BUILDING_WITH_FAST_CGI)
+	      && FCGX_IsCGI()
+#endif
+	      )
 	    usage ();
 	}
     }
@@ -551,6 +583,33 @@ engine_make_request (int fd)
   pagefunc_set_variable ("env::remote_ident", temp);
   pagefunc_set_variable ("mhtml::remote-ident", temp);
 
+#if defined (BUILDING_WITH_FAST_CGI)
+  /* Now set our local variables. */
+  temp = (char *)getenv ("SERVER_NAME");
+
+  if (temp != (char *)NULL)
+    {
+      http_host = strdup (temp);
+    }
+  else
+    {
+      char buffer[100];
+
+      strcpy (buffer, "www.nohost.net");
+      gethostname (buffer, 100);
+      http_host = strdup (buffer);
+    }
+  pagefunc_set_variable ("mhtml::server-name", http_host);
+
+  temp = (char *)getenv ("SERVER_PORT");
+  if (temp == (char *)NULL)
+    temp = "80";
+
+  http_port = atoi (temp);
+  pagefunc_set_variable ("mhtml::server-port", temp);
+
+#endif /* BUILDING_WITH_FAST_CGI */
+
   /* Build a reasonable copy of the Mime headers that we might expect to be
      present. */
   {
@@ -593,14 +652,39 @@ engine_make_request (int fd)
        engine to randomly assume that the browser is cookie compatible,
        especially when it isn't! */
     temp = (char *)getenv ("METAHTML_URL_COOKIE");
+    if (temp == (char *)NULL)
+      temp = (char *)getenv ("HTTP_METAHTML_URL_COOKIE");
     if (temp != (char *)NULL)
       bprintf (headers, "URL-Cookie: SID=%s\n", temp);
 
     /* Content-Length */
     temp = (char *)getenv ("CONTENT_LENGTH");
     if (temp != (char *)NULL)
-      bprintf (headers, "Content-length: %s\n", temp);
+      {
+	bprintf (headers, "Content-length: %s\n", temp);
+#if (BUILDING_WITH_FAST_CGI)
+	/* When we are using the losing OpenMarket FCGI library, there
+	   isn't a single clean way to simply pass a file descriptor off
+	   to our previously build libraries.  This just totally sucks.
 
+	   If we pass off the socket file descriptor, it is possible that
+	   the over-zealous buffering of the Open Market code will have
+	   already read data into a buffer, and we won't have access to it.
+
+	   This means that for the case of Fast CGI, we cannot use the
+	   existing file descriptor.  Great.  I'll just yet one more
+	   level of indirection here.  Fucking lossage big time. */
+  	fast_cgi_content_length = atoi (temp);
+	if (fast_cgi_content_length)
+	  {
+	    fast_cgi_content = (unsigned char *)
+	      xmalloc (1 + fast_cgi_content_length);
+
+	    fread (fast_cgi_content, 1, fast_cgi_content_length, stdin);
+	  }
+#endif /* BUILDING_WITH_FAST_CGI */
+      }
+	
     /* Content-Type */
     temp = (char *)getenv ("CONTENT_TYPE");
     if (temp != (char *)NULL)
@@ -643,10 +727,14 @@ process_request (void)
 
       if (mhttpd_page_redirect_p (perfun))
 	{
-	  if (!strncasecmp (perfun->buffer, "HTTP/", 5) == 0)
-	    bprintf_insert (perfun, 0, "HTTP/1.0 302 Found\n");
+	  if (nph)
+	    {
+	      if (!strncasecmp (perfun->buffer, "HTTP/", 5) == 0)
+		bprintf_insert (perfun, 0, "HTTP/1.0 302 Found\n");
+	    }
 
-	  mhttpd_write (mhtml_stdout_fileno, perfun->buffer, perfun->bindex);
+	  fwrite (perfun->buffer, perfun->bindex, 1, stdout);
+	  fflush (stdout);
 	  return;
 	}
 
@@ -673,7 +761,7 @@ process_request (void)
 	      }
 	}
 
-      mhttpd_write (mhtml_stdout_fileno, result->page->buffer + i,
-		    result->page->bindex - i);
+      fwrite (result->page->buffer + i, result->page->bindex - i, 1, stdout);
+      fflush (stdout);
     }
 }
