@@ -28,61 +28,18 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
 #define NEW_CGI_EXEC 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <sys/types.h>
-#include <errno.h>
+#include "language.h"
 
 #if defined (NEW_CGI_EXEC)
-#  if defined (sgi)
-#    include <bstring.h>
-#    define vfork fork
-#  endif
-
 #  if defined (__WINNT__)
-/* Needed for select(). */
+     /* Needed for select(). */
 #    include <sys/socket.h>
-#    define vfork fork
 #  endif /* __WINNT__ */
-
-#  include <sys/wait.h>
-#  include <sys/time.h>
-#  include <signal.h>
-#endif /* NEW_CGI_EXEC */
-
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <fcntl.h>
+#  if defined (HAVE_VFORK_H)
+#    include <vfork.h>
 #  endif
-#endif /* !Solaris */
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <signal.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "forms.h"
-#include "session.h"
-#include "pages.h"
-#include "parser.h"
-
-#if !defined (errno)
-extern int errno;
-#endif
+#endif /* NEW_CGI_EXEC */
 
 PACKAGE_INITIALIZER (initialize_cgi_exec_functions)
 
@@ -108,10 +65,6 @@ initialize_cgi_exec_functions (Package *package)
 }
 
 #if defined (NEW_CGI_EXEC)
-#if defined (sparc) || defined (__linux__) || defined (sgi) || defined (Solaris) || defined (__WINNT__) || defined (hpux)
-typedef void (*sig_t) (int);
-#endif
-
 static char **
 make_environ (void)
 {
@@ -207,7 +160,7 @@ gather_input (int fd)
         while (!done)
           {
             while ((bindex + 1024) > bsize)
-              buffer = (char *)xrealloc (buffer, (bsize += 1024));
+              buffer = (char *)xrealloc (buffer, (bsize += 1025));
             buffer[bindex] = '\0';
 
             amount_read = read (fd, buffer + bindex, 1023);
@@ -246,7 +199,11 @@ shell_execute (char *command, char **output_textp, char **errors_textp)
 
   if (errors_text == (char *)NULL)
     {
+#if defined (__WINNT__)
+      char *physical_path = "/www/bin/bash.exe";
+#else
       char *physical_path = "/bin/sh";
+#endif
 
       if (errors_text == (char *)NULL)
 	{
@@ -301,6 +258,8 @@ shell_execute (char *command, char **output_textp, char **errors_textp)
 	      close (0);
 	      dup2 (stdout_pipe[1], 1);
 	      dup2 (stderr_pipe[1], 2);
+	      close (stdout_pipe[1]);
+	      close (stderr_pipe[1]);
 
 	      execve (physical_path, argv, make_environ ());
 	      _exit (127);
