@@ -28,28 +28,7 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#include <signal.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "forms.h"
-#include "session.h"
-#include "pages.h"
-#include "parser.h"
+#include "language.h"
 #include <des.h>
 
 static void pf_encrypt (PFunArgs);
