@@ -28,58 +28,7 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <regex.h>
-#include <setjmp.h>
-#include <sys/types.h>
-#include <errno.h>
-#include <time.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#endif /* !Solaris */
-
-#include <locking.h>
-#if !defined (HAVE_SRANDOM)
-#  include <math.h>
-#  define srandom(seed) srand (seed)
-#  define random() rand()
-#endif
-
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include <wisper/wisp.h>
-#include "forms.h"
-#include "session.h"
-#include "pages.h"
-#include "parser.h"
-
-extern jmp_buf page_jmp_buffer;
-
-#if defined (macintosh)
-extern char *strdup (const char *string);
-#  include <sys/fcntl.h>
-#  define os_open(name, flags, mode) open (name, flags)
-#else
-#  define os_open(name, flags, mode) open (name, flags, mode)
-#endif
-
-#if !defined (errno)
-extern int errno;
-#endif
+#include "language.h"
 
 static void pf_with_open_database (PFunArgs);
 static void pf_database_load_record (PFunArgs);
@@ -195,19 +144,6 @@ db_mode (char *modename)
   return (mode);
 }
 
-static char *
-db_lockname (char *dbname)
-{
-  static char lockname[1024];
-
-  if ((dbname == (char *)NULL) || (strlen (dbname) > 1018))
-    return ((char *)NULL);
-
-  strcpy (lockname, dbname);
-  strcat (lockname, ".LCK");
-  return (lockname);
-}
-
 static void
 lock_database (char *dbname, int open_mode, int *lock, DBFILE *db)
 {
@@ -236,7 +172,8 @@ lock_database (char *dbname, int open_mode, int *lock, DBFILE *db)
     {
       if ((*db = database_open (dbname, open_mode)) == (DBFILE)0)
 	{
-	  page_syserr ("%s: %s", dbname, database_strerror ());
+	  if (database_had_error_p ())
+	    page_syserr ("%s: %s", dbname, database_strerror ());
 	  /* unlink (lockname); */
 	  UNLOCKFILE (fd);
 	  close (fd);
@@ -254,8 +191,8 @@ unlock_database (char *dbname, int *lock, DBFILE *db)
 
   if (*lock > -1)
     {
-      char *lockname = db_lockname (dbname);
-      unlink (lockname);
+      /* char *lockname = db_lockname (dbname);
+	 unlink (lockname); */
       UNLOCKFILE (*lock);
       close (*lock);
     }
@@ -662,11 +599,11 @@ pf_database_save_package (PFunArgs)
 
 	    if (database_had_error_p ())
 	      page_syserr ("%s", database_strerror ());
+	    else
+	      result = "true";
 	  }
 
 	  bprintf_free_buffer (data);
-
-	  result = "true";
 	}
 
       dbobj_free (key);
