@@ -27,52 +27,7 @@
    129 El Paseo Court
    Santa Barbara, CA
    93101  */
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <regex.h>
-#include <setjmp.h>
-#include <sys/types.h>
-#include <errno.h>
-#include <time.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#endif /* !Solaris */
-
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include <wisper/wisp.h>
-#include <locking.h>
-#include "forms.h"
-#include "session.h"
-#include "pages.h"
-#include "parser.h"
-
-extern jmp_buf page_jmp_buffer;
-
-#if defined (macintosh)
-extern char *strdup (const char *string);
-#  include <sys/fcntl.h>
-#  define os_open(name, flags, mode) open (name, flags)
-#else
-#  define os_open(name, flags, mode) open (name, flags, mode)
-#endif
-
-#if !defined (errno)
-extern int errno;
-#endif
+#include "language.h"
 
 static void pf_with_open_database (PFunArgs);
 static void pf_get_var (PFunArgs);
@@ -141,19 +96,6 @@ db_mode (char *modename)
   return (mode);
 }
 
-static char *
-db_lockname (char *dbname)
-{
-  static char lockname[1024];
-
-  if ((dbname == (char *)NULL) || (strlen (dbname) > 1018))
-    return ((char *)NULL);
-
-  strcpy (lockname, dbname);
-  strcat (lockname, ".LCK");
-  return (lockname);
-}
-
 static void
 lock_database (char *dbname, int open_mode, int *lock, DBFILE *db)
 {
