@@ -348,10 +348,9 @@ static void
 cleanup (char *string)
 {
   register int i, j, len;
-  char *dest;
+  char *dest = string;
 
   len = strlen (string);
-  dest = (char *)alloca (1 + len);
 
   for (i = 0, j = 0; i < len; i++)
     {
@@ -372,7 +371,6 @@ cleanup (char *string)
     }
 
   dest[j] = '\0';
-  strcpy (string, dest);
 }
 
 static int
