@@ -30,65 +30,26 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <regex.h>
-#include <setjmp.h>
-#include <sys/types.h>
-#include <time.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#endif /* !Solaris */
-
-#include <locking.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include <wisper/wisp.h>
-#include "forms.h"
-#include "pages.h"
-#include "parser.h"
-
-extern jmp_buf page_jmp_buffer;
-
-#if defined (macintosh)
-extern char *strdup (const char *string);
-#  include <sys/fcntl.h>
-#  define os_open(name, flags, mode) open (name, flags)
-#else
-#  define os_open(name, flags, mode) open (name, flags, mode)
-#endif
-
-#if !defined (xfree)
-#  define xfree(x) if (x) free (x)
-#endif
+#include "language.h"
 
 static void pf_with_open_database (PFunArgs);
 static void pf_database_query (PFunArgs);
 static void pf_database_exec_query (PFunArgs);
+static void pf_database_exec_sql (PFunArgs);
 static void pf_database_next_record (PFunArgs);
 static void pf_host_databases (PFunArgs);
 static void pf_database_tables (PFunArgs);
-static void pf_database_table_fields (PFunArgs);
+static void pf_database_tables_info (PFunArgs);
+static void pf_database_columns (PFunArgs);
+static void pf_database_column_info (PFunArgs);
+static void pf_database_columns_info (PFunArgs);
+static void pf_database_query_info (PFunArgs);
 static void pf_database_num_rows (PFunArgs);
 static void pf_database_set_pos (PFunArgs);
 static void pf_database_save_record (PFunArgs);
 static void pf_database_load_record (PFunArgs);
 static void pf_database_delete_record (PFunArgs);
 static void pf_database_save_package (PFunArgs);
-static void pf_database_field_info (PFunArgs);
 static void pf_database_set_options (PFunArgs);
 static void pf_cursor_get_column (PFunArgs);
 static void pf_query_get_column (PFunArgs);
@@ -112,6 +73,14 @@ static void pf_sql_transact (PFunArgs);
 #define GSQL_DOUBLE			8
 #define GSQL_VARCHAR 			12
 
+/* These are not standard ANSI */
+#define GSQL_IDENT          100
+#define GSQL_NULL           101
+#define GSQL_IDX            102
+#define GSQL_SYSVAR         103
+#define GSQL_UINT           104
+#define GSQL_ANY            105
+
 /* Reporting of db errors to Meta-HTML environment  */
 #define GSQL_DEFAULT_ERRMSG NULL
 
@@ -317,13 +286,10 @@ free_cursor (DBCursor *cursor)
     gsql_free_result (cursor->result);
 }
 
-/* Dereference a Meta-HTML variable which indexes a cursor object.
-   Takes the high byte, and computes an index into the open databases,
-   and then uses the low byte to index into that database's cursor list. */
 static DBCursor *
-get_cursor_ref (Package *vars)
+lookup_cursor_ref (char *cursor_ref)
 {
-  char *cursor_ref = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
   Database * db;
   int cursor_token = -1;		/* The combined db::cursor ids */
   int cursor_id;
@@ -344,8 +310,6 @@ get_cursor_ref (Package *vars)
 	}
     }
 
-  xfree (cursor_ref);
-
   if (cursor_token == -1)
     return ((DBCursor *) NULL);
 
@@ -364,6 +328,21 @@ get_cursor_ref (Package *vars)
     return ((DBCursor *) NULL);
 }
 
+
+/* Dereference a Meta-HTML variable which indexes a cursor object.
+   Takes the high byte, and computes an index into the open databases,
+   and then uses the low byte to index into that database's cursor list. */
+static DBCursor *
+get_cursor_ref (Package *vars)
+{
+  DBCursor *cursor;
+  char *cursor_ref = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  cursor = lookup_cursor_ref (cursor_ref);
+  xfree (cursor_ref);
+  return cursor;
+}
+
+
 /* *************************************************************** */
 /*								   */
 /*		     Database Manipulation Functions		   */
@@ -372,58 +351,6 @@ get_cursor_ref (Package *vars)
 
 static int database_environment_level = 0;
 
-static char *hexchars = "0123456789ABCDEF";
-
-/* Generate a 16 byte printable hashcode of a string, using hex digits.
-   Always returns a newly consed string. */
-static char *
-hash_string (char *src)
-{
-  register int i, c;
-  register char *p;
-  char buf[64];
-  char hashbuf[9];
-
-  memset (hashbuf, 0, 9);
-
-  if (strlen (src) <= 8)
-    {
-      strcpy (hashbuf, src);
-    }
-  else
-    {
-      /* Do a linear feedback shiftregister. */
-      for (i = 0; i < 8; i++)
-	hashbuf[i] = src[i];
-
-      p = src + i;
-      while (1)
-	{
-	  char tmp = hashbuf[0];
-
-	  c = *p++;
-	  if (c == 0)
-	    break;
-
-	  hashbuf[0] = hashbuf[1] ^ c;
-	  hashbuf[1] = hashbuf[2];
-	  hashbuf[2] = hashbuf[3] ^ tmp;
-	  hashbuf[3] = hashbuf[4] ^ ~c;
-	  hashbuf[4] = hashbuf[5] ^ hashbuf[2] ^ tmp;
-	  hashbuf[5] = hashbuf[6];
-	  hashbuf[6] = hashbuf[7] ^ c ^ tmp;
-	}
-    }
-
-  for (p = buf, i = 0; i < 8; i ++)
-    {
-      *p++ = hexchars[(hashbuf[i] >> 4) & 0x0F];
-      *p++ = hexchars[(hashbuf[i] & 0x0F)];
-    }
-  *p = 0;
-  return (strdup (buf));
-}
-
 /* We pass an string which contains a numeric representation of the
    SQL escape code character, such as "39" */
 static void
@@ -489,27 +416,6 @@ gsql_database_sql_prefix_tablenames (Database *db)
   return (db->sql_prefix_tablenames);
 }
 
-static char *
-db_lockname (char *dsn)
-{
-  static char lockname[1024];
-  char *result = (char *)NULL;
-  char *hashval;
-
-  if (dsn != (char *)NULL)
-    {
-      hashval = hash_string (dsn);
-      strcpy (lockname, "/tmp/mhtmldb");
-      strcat (lockname, "-");
-      strcat (lockname, hashval);
-      strcat (lockname, ".LCK");
-      free (hashval);
-      result = lockname;
-    }
-
-  return (lockname);
-}
-
 static void
 lock_database (char *dsn, int *lock, Database *db, int lock_p)
 {
@@ -740,6 +646,45 @@ pf_with_open_database (PFunArgs)
     longjmp (page_jmp_buffer, 1);
 }
 
+/* <database-exec-sql dbvar query-string>
+   Executes an SQL query, within the scope of with-open-database.
+
+   returns TRUE if no error (GSQL_ERROR), NULL otherwise.
+*/
+
+static void
+pf_database_exec_sql (PFunArgs)
+{
+  int dbindex;
+
+  if (database_environment_level != 0)
+    {
+      Database *db = get_dbref_internal (vars, &dbindex);
+
+      if (db != (Database *)NULL)
+	{
+	  char *dbquery;
+
+	  dbquery = mhtml_evaluate_string (get_positional_arg (vars, 1));
+
+	  if (dbquery != (char *)NULL)
+	    {
+	      if (gsql_query (db, dbquery) == GSQL_ERROR)
+		{
+		  gsql_save_error_message (db, GSQL_DEFAULT_ERRMSG);
+		}
+	      else
+		{
+		  bprintf_insert (page, start, "true");
+		  *newstart += 4;
+		}
+	      free (dbquery);
+	    }
+	}
+    }
+}
+
+
 /* <database-exec-query dbvar query-string [cursor=cursor]>
    Executes an SQL query, within the scope of with-open-database.
 
@@ -819,8 +764,6 @@ pf_database_num_rows (PFunArgs)
 {
   DBCursor *cursor = get_cursor_ref (vars);
 
-  cursor = get_cursor_ref (vars);
-
   if ((cursor != (DBCursor *)NULL) && (cursor->result != (gsql_result *)NULL))
     {
       int nrows = gsql_number_of_rows (cursor->result);
@@ -1003,8 +946,75 @@ split_colnames (char *string, int *ncols)
   return (result);
 }
 
+static char *
+alist_of_field_info (gsql_field *field, Database *db);
+
+
+/* <database-query-info cursor [result=varname]> 
+
+   Returns an array of alists of information about the columns of the
+   given cursor's query result set.
+
+ */
+static void
+pf_database_query_info (PFunArgs)
+{
+  if (database_environment_level != 0)
+    {
+      DBCursor *cursor = get_cursor_ref (vars);
+
+      if (cursor != (DBCursor *) NULL)
+	{
+	  gsql_result *result = cursor->result;
+
+	  if (gsql_database_connected (cursor->db) &&
+	      (result != (gsql_result *)NULL))
+	    {
+	      int i;
+	      int cols = gsql_num_fields (result);
+	      char *varname;
+	      char **colnames;
+
+	      /* The name of the variable to stuff the results into. */
+	      varname = mhtml_evaluate_string (get_value (vars, "RESULT"));
+	      colnames = (char **)xmalloc ((cols + 1) * sizeof (char *));
+
+	      /* Reset the field descriptor cursor to start of record */
+
+	      for (i = 0; i < cols; i++)
+		{
+		  gsql_field *field = gsql_fetch_field (result, i);
+
+		  colnames[i] = alist_of_field_info (field, cursor->db);
+		}
+	      colnames[i] = (char *) NULL;
+
+	      if (!empty_string_p (varname))
+		{
+		  symbol_store_array (varname, colnames);
+		}
+	      else
+		{
+		  for (i = 0; i < cols; i++)
+		    {
+		      bprintf_insert (page, start, "%s\n", colnames[i]);
+		      start += 1 + strlen (colnames[i]);
+		      free (colnames[i]);
+		    }
+		  free (colnames);
+
+		  *newstart = start;
+		}
+
+	    }
+	}
+    }
+}
+
+
+
 /* <database-next-record cursor [package=PKGNAME] [colnames=name1,name2.]
-                                [prefixtablenames=true]>
+                                [prefixtablenames=true] [appendvars=true]>
    This should be called after a call to database-exec-query.
    It fetches the next result row, and binds all the fields of the
    result row to variables in the current package, or in PKGNAME
@@ -1019,6 +1029,9 @@ split_colnames (char *string, int *ncols)
    sequentially to these names instead of the field names in the
    result set.
 
+   If appendvars is supplied, then values are appended to variables,
+   to form an array, rather than overwriting the previous values.
+
    Return "true" if successful, or "" if not. */
 static void
 pf_database_next_record (PFunArgs)
@@ -1040,6 +1053,8 @@ pf_database_next_record (PFunArgs)
 		(get_one_of (vars, "PACKAGE", "PREFIX", (char *)NULL));
 	      char *prefixtablenames =
 		mhtml_evaluate_string (get_value (vars, "prefixtablenames"));
+	      char *appendvars =
+		mhtml_evaluate_string (get_value (vars, "appendvars"));
 	      char **fieldnames = (char **)NULL;
 	      char *prev_fieldname = (char *) NULL;
 	      char *fieldname= (char *) NULL;
@@ -1056,6 +1071,53 @@ pf_database_next_record (PFunArgs)
 		xfree (t);
 	      }
 
+
+	      /* If the user does not want variables to have their
+		 appended values persist across calls to next-record,
+		 then we loop here to null out all the symbols which
+		 are going to be set in this result row. */
+	      if (empty_string_p (appendvars))
+		{
+		  for (i = 0; i < limit; i++)		  
+		    {
+		      gsql_field *field = gsql_fetch_field (query_result, i);
+		      char buf[256];
+		      Symbol *sym;
+		      
+		      /* If the user supplied column names, use them. */
+		      if (fieldnames != (char **)NULL)
+			{
+			  if (i < ncolnames)
+			    fieldname = fieldnames[i];
+			  else
+			    fieldname = (char *) NULL;
+			}
+		      else
+			fieldname = gsql_field_name (field);
+
+		      if (empty_string_p (fieldname))
+			break;
+		      else
+			{
+			  if (gsql_field_table (field)
+			      && (gsql_database_sql_prefix_tablenames (cursor->db)
+				  || !(empty_string_p (prefixtablenames))))
+			    {
+			      strcpy (buf, gsql_field_table (field));
+			      strcat (buf, ".");
+			      strcat (buf, fieldname);
+			    }
+			  else
+			    {
+			      strcpy (buf, fieldname);
+			    }
+
+			  sym = symbol_intern_in_package (package, buf);
+			  symbol_reset (sym);
+			}
+		    }
+		}
+
 	      /* Loop over fields of the row, binding variables.
 		 All data is already coerced to strings, so we
 		 do not do any type conversion. */
@@ -1105,7 +1167,7 @@ pf_database_next_record (PFunArgs)
 
 		      sym = symbol_intern_in_package (package, buf);
 		      value = gsql_get_column (query_result, i);
-		      symbol_add_value (sym, value);
+		      symbol_add_value (sym, value ? value : "");
 
 		      xfree (value);
 		    }
@@ -1173,11 +1235,22 @@ pf_database_set_options (PFunArgs)
    with a database already open, i.e., within the scope of a
    <with-open-database>.
 
-   Args: <msql::database-tables dbvar [result=VAR]>
+   Args: <odbc::database-tables dbvar [result=VAR] 
+            [tablequalifier=VAL]
+	    [tableowner=VAL]
+	    [tablename=VAL]
+	    [tabletype=VAL]>
+
    Returns resulting array, or places it into VAR if supplied. */
+
 static void
 pf_database_tables (PFunArgs)
 {
+  char *qualifier = mhtml_evaluate_string (get_value (vars, "tablequalifier"));
+  char *owner     = mhtml_evaluate_string (get_value (vars, "tableowner"));
+  char *name      = mhtml_evaluate_string (get_value (vars, "tablename"));
+  char *type      = mhtml_evaluate_string (get_value (vars, "tabletype"));
+
   if (database_environment_level != 0)
     {
       Database *db = get_dbref (vars);
@@ -1185,7 +1258,8 @@ pf_database_tables (PFunArgs)
       if ((db != (Database *)NULL) && gsql_database_connected (db))
 	{
 	  char *resultvar = mhtml_evaluate_string (get_value (vars, "RESULT"));
-	  gsql_result *result = gsql_db_list_tables (db);
+	  gsql_result *result = gsql_db_list_tables
+	                            (db, qualifier, owner,name, type);
 
 	  if (result != (gsql_result *)NULL)
 	    {
@@ -1231,6 +1305,8 @@ pf_database_tables (PFunArgs)
 		}
 	      gsql_free_result (result);
 	    }
+
+	  xfree (resultvar);
 	}
     }
 }
@@ -1239,11 +1315,11 @@ pf_database_tables (PFunArgs)
    with a database already open, i.e., within the scope of a
    <msql::with-open-database>.
 
-   Args: <msql::database-table-fields dbvar tablename [result=VAR]>
+   Args: <msql::database-columns dbvar tablename [result=VAR]>
    Returns an array of column names, or sets the value of VAR
    if supplied. */
 static void
-pf_database_table_fields (PFunArgs)
+pf_database_columns (PFunArgs)
 {
   /* No errors yet! */
   gsql_clear_error_message ();
@@ -1309,8 +1385,7 @@ pf_database_table_fields (PFunArgs)
     }
 }
 
-/* Read the record in DB at KEY, and return a package containing the
-   fields and values.
+/* Convert a gsql_result struct into a meta-html package.
 
    If colnames[] is not null, read sequential column names from it
    instead of from the result set. colnames[] is a null terminated
@@ -1381,7 +1456,7 @@ package_from_row (gsql_result *query_result,
 
 	  value = gsql_get_column (query_result, i);
 	  sym = symbol_intern_in_package (package, buf);
-	  symbol_add_value (sym, value);
+	  symbol_add_value (sym, value ? value : "");
 	}
 
       xfree (value);
@@ -1694,7 +1769,7 @@ bprintf_gsql_escape (BPRINTF_BUFFER *out, char *src, gsql_field *field,
       i++;
       if ((maxlength != 0) && (i > maxlength)) return;
 
-      if (c == '\'')
+      if ((c == '\'') || (c == '(') || (c == ')'))
 	bprintf (out, escape_string);
 
       bprintf (out, "%c",c);
@@ -2200,11 +2275,11 @@ pf_database_save_package (PFunArgs)
   char *method   = mhtml_evaluate_string (get_value (vars, "method"));
   BPRINTF_BUFFER *query = (BPRINTF_BUFFER *) NULL;
   char *result = (char *)NULL;
-  int status;
   gsql_result *table_fields = (gsql_result *) NULL;
-  gsql_field *field;
-  gsql_field *keyfield;
+  gsql_field *field = (gsql_field *)NULL;
+  gsql_field *keyfield = (gsql_field *)NULL;
   Symbol **symbols = (Symbol **)NULL;
+  int status = 0;
   int items_printed = 0;
   int errors_found = 0;
 
@@ -2328,9 +2403,7 @@ pf_database_save_package (PFunArgs)
 		      else
 			{
 			  register int j;
-			  BPRINTF_BUFFER *tmpbuf;
-
-			  tmpbuf = bprintf_create_buffer ();
+			  BPRINTF_BUFFER *tmpbuf = bprintf_create_buffer ();
 
 			  for (j = 0; j < sym->values_index; j++)
 			    bprintf (tmpbuf, sym->values[j]);
@@ -2411,9 +2484,8 @@ pf_database_save_package (PFunArgs)
 			  /* Collect up the array values
 			     into a string. */
 			  register int j;
-			  BPRINTF_BUFFER *tmpbuf;
+			  BPRINTF_BUFFER *tmpbuf = bprintf_create_buffer ();
 
-			  tmpbuf = bprintf_create_buffer ();
 			  for (j = 0; j < sym->values_index; j++)
 			    bprintf (tmpbuf, sym->values[j]);
 
@@ -2465,15 +2537,23 @@ pretty_field_type (int fieldtype)
 {
   switch (fieldtype)
     {
-    case  GSQL_CHAR: return "GSQL_CHAR";
-    case  GSQL_NUMERIC: return "GSQL_NUMERIC";
-    case  GSQL_DECIMAL: return "GSQL_DECIMAL";
-    case  GSQL_INTEGER: return "GSQL_INTEGER";
+    case  GSQL_CHAR:     return "GSQL_CHAR";
+    case  GSQL_NUMERIC:  return "GSQL_NUMERIC";
+    case  GSQL_DECIMAL:  return "GSQL_DECIMAL";
+    case  GSQL_INTEGER:  return "GSQL_INTEGER";
     case  GSQL_SMALLINT: return "GSQL_SMALLINT";
-    case  GSQL_FLOAT: return "GSQL_FLOAT";
-    case  GSQL_REAL: return "GSQL_REAL";
-    case  GSQL_DOUBLE: return "GSQL_DOUBLE";
-    case  GSQL_VARCHAR: return "GSQL_VARCHAR";
+    case  GSQL_FLOAT:    return "GSQL_FLOAT";
+    case  GSQL_REAL:     return "GSQL_REAL";
+    case  GSQL_DOUBLE:   return "GSQL_DOUBLE";
+    case  GSQL_VARCHAR:  return "GSQL_VARCHAR";
+
+    case GSQL_IDENT:     return "GSQL_IDENT";
+    case GSQL_NULL:      return "GSQL_NULL";
+    case GSQL_IDX:       return "GSQL_IDX";
+    case GSQL_SYSVAR:    return "GSQL_SYSVAR";
+    case GSQL_UINT:      return "GSQL_UINT";
+    case GSQL_ANY:       return "GSQL_ANY";
+      
     default:
       return "GSQL_CHAR";
     }
@@ -2486,18 +2566,92 @@ string_from_boolean (int bool)
   else return "true";
 }
 
-/* <database-field-info dbvar table fieldname>
-
-   Returns an alist of field properties:
-   ((name . "column_name") (length . "max byte length") (type . "sqlDatatype")
-    (is_primary_key . "true") (is_not_null . "true"))
-
- */
-static void
-pf_database_field_info (PFunArgs)
+/* Create a Meta-HTML string representation of an alist, which contains
+   table field information. 
+   */
+static char *
+alist_of_field_info (gsql_field *field, Database *db)
 {
-  BPRINTF_BUFFER *tmpbuf;
+  BPRINTF_BUFFER *tmpbuf = (BPRINTF_BUFFER *)NULL;
+  char *val = (char *)NULL;
 
+  if ((db != (Database *)NULL) && gsql_database_connected (db))
+    {
+      tmpbuf = bprintf_create_buffer ();
+
+      bprintf (tmpbuf, "(");
+      bprintf (tmpbuf, "(NAME . \"%s\")",
+	       gsql_field_name (field));
+
+      bprintf (tmpbuf, "(LENGTH . \"%d\")",
+	       gsql_field_length (field));
+
+      bprintf (tmpbuf, "(TYPE . \"%s\")",
+	       pretty_field_type (gsql_field_type (field)));
+
+      bprintf (tmpbuf, "(IS_UNIQUE . \"%s\")",
+	       string_from_boolean
+	       (gsql_field_is_unique (field)));
+
+      bprintf (tmpbuf, "(IS_NULLABLE . \"%s\")",
+	       string_from_boolean
+	       (! (gsql_field_is_not_null (field))));
+
+      bprintf (tmpbuf, "(QUALIFIER . \"%s\")",
+	       gsql_field_qualifier (field));
+
+      bprintf (tmpbuf, "(OWNER . \"%s\")",
+	       gsql_field_owner (field));
+
+      bprintf (tmpbuf, "(TYPENAME . \"%s\")",
+	       gsql_field_typename (field));
+
+      bprintf (tmpbuf, "(PRECISION . \"%d\")",
+	       gsql_field_precision (field));
+
+      bprintf (tmpbuf, "(SCALE . \"%d\")",
+	       gsql_field_scale (field));
+
+      bprintf (tmpbuf, "(RADIX . \"%d\")",
+	       gsql_field_radix (field));
+
+      bprintf (tmpbuf, "(TABLE . \"%s\")",
+	       gsql_field_table (field));
+
+      bprintf (tmpbuf, ")");
+    }
+  
+  if (tmpbuf != (BPRINTF_BUFFER *)NULL)
+    {
+      if (tmpbuf->buffer != (char *)NULL)
+	val = strdup (tmpbuf->buffer);
+      else
+	val = (char *)NULL;
+
+      bprintf_free_buffer (tmpbuf);
+    }
+
+  return (val);
+}
+
+
+/* List all the column information for a table. This must be done
+   with a database already open, i.e., within the scope of a
+   <msql::with-open-database>.
+
+   Args: <msql::database-columns-info dbvar tablename [result=VAR]>
+   Returns an array of alists, one per table column, 
+   or sets the value of VAR if supplied. 
+
+   Each alist contains 
+   ((name . "column_name") (length . "max byte length") (type . "sqlDatatype")
+    (is_unique . "true") (is_not_null . "true"))
+
+*/
+
+static void
+pf_database_columns_info (PFunArgs)
+{
   /* No errors yet! */
   gsql_clear_error_message ();
 
@@ -2508,7 +2662,88 @@ pf_database_field_info (PFunArgs)
       if ((db != (Database *)NULL) && gsql_database_connected (db))
 	{
 	  char *tablename;
-	  char *fieldname;
+
+	  /* The name of the table we are enquiring about. */
+	  tablename = mhtml_evaluate_string (get_positional_arg (vars, 1));
+
+	  if (!empty_string_p (tablename))
+	    {
+	      gsql_result *result = gsql_list_fields (db, tablename);
+
+	      if (result != (gsql_result *)NULL)
+		{
+		  register int i;
+		  int cols = gsql_num_fields (result);
+		  char *varname;
+		  char **colnames;
+
+		  /* The name of the variable to stuff the results into. */
+		  varname = mhtml_evaluate_string (get_value (vars, "RESULT"));
+		  colnames = (char **)xmalloc ((cols + 1) * sizeof (char *));
+
+		  /* Reset the field descriptor cursor to start of record */
+
+		  for (i = 0; i < cols; i++)
+		    {
+		      gsql_field *field = gsql_fetch_field (result, i);
+
+		      colnames[i] = alist_of_field_info (field, db);
+		    }
+		  colnames[i] = (char *) NULL;
+
+		  if (!empty_string_p (varname))
+		    {
+		      symbol_store_array (varname, colnames);
+		    }
+		  else
+		    {
+		      for (i = 0; i < cols; i++)
+			{
+			  bprintf_insert (page, start, "%s\n", colnames[i]);
+			  start += 1 + strlen (colnames[i]);
+			  free (colnames[i]);
+			}
+		      free (colnames);
+
+		      *newstart = start;
+		    }
+
+		  xfree (varname);
+		}
+	    }
+	  xfree (tablename);
+	}
+    }
+}
+
+
+/* <database-column-info dbvar table fieldname>
+
+   Returns an alist of properties of a column of a table:
+   ((name . "column_name") (length . "max byte length") (type . "sqlDatatype")
+    (is_unique . "true") (is_not_null . "true"))
+
+    This is actually implemented by fetching information on *all*
+    columns and selecting the specified one. It is thus more
+    efficient, if you are looking for info on several columns, to use
+    database-columns-info, and get all the column information in a
+    single array.
+
+ */
+static void
+pf_database_column_info (PFunArgs)
+{
+  /* No errors yet! */
+  gsql_clear_error_message ();
+
+  if (database_environment_level != 0)
+    {
+      Database *db = get_dbref (vars);
+
+      if ((db != (Database *)NULL) && gsql_database_connected (db))
+	{
+	  char *tablename = (char *)NULL;
+	  char *fieldname = (char *)NULL;
 
 	  /* The name of the table we are inquiring about. */
 	  tablename = mhtml_evaluate_string (get_positional_arg (vars, 1));
@@ -2522,9 +2757,7 @@ pf_database_field_info (PFunArgs)
 		{
 		  register int i;
 		  int cols = gsql_num_fields (result);
-		  int found = 0;
-
-		  tmpbuf = bprintf_create_buffer ();
+		  char *alist = (char *)NULL;
 
 		  for (i = 0; i < cols; i++)
 		    {
@@ -2532,36 +2765,17 @@ pf_database_field_info (PFunArgs)
 
 		      if (strcasecmp (gsql_field_name (field), fieldname) == 0)
 			{
-			  bprintf (tmpbuf, "(");
-			  bprintf (tmpbuf, "(NAME . \"%s\")",
-				   gsql_field_name (field));
-			  bprintf (tmpbuf, "(LENGTH . \"%d\")",
-				   gsql_field_length (field));
-
-			  bprintf
-			    (tmpbuf, "(TYPE . \"%s\")",
-			     pretty_field_type (gsql_field_type (field)));
-
-			  bprintf (tmpbuf, "(IS_PRIMARY_KEY . \"%s\")",
-				   string_from_boolean
-				   (gsql_field_is_primary_key (field)));
-			  bprintf (tmpbuf, "(IS_NULLABLE . \"%s\")",
-				   string_from_boolean
-				   (! (gsql_field_is_not_null (field))));
-			  bprintf (tmpbuf, ")");
-			  found = 1;
+			  alist = alist_of_field_info (field, db);
 			  break;
 			}
 		    }
 
-		  if (found)
+		  if (alist != (char *)NULL)
 		    {
-		      bprintf_insert (page, start, "%s", tmpbuf->buffer);
-		      *newstart += tmpbuf->bindex;
+		      bprintf_insert (page, start, "%s", alist);
+		      *newstart += strlen (alist);
+		      free (alist);
 		    }
-
-		  bprintf_free_buffer (tmpbuf);
-
 		}
 	    }
 	  xfree (tablename);
@@ -2608,3 +2822,139 @@ pf_sql_transact (PFunArgs)
     }
   xfree (action_arg);
 }
+
+/* Convert a gsql_result struct into a meta-html alist.
+
+ */
+static char *
+alist_from_row (gsql_result *query_result)
+{
+  register int i;
+  int num_fields = 0;
+  char *fieldname = (char *) NULL;
+  char *val;
+  BPRINTF_BUFFER *alist;
+
+  alist = bprintf_create_buffer ();
+  /* Loop over fields of the row, adding alist entries.  All results
+     should already be strings, so we don't need to worry about
+     converting INT, REAL, or other datatypes to strings. */
+
+  num_fields = gsql_num_fields (query_result);
+
+  bprintf (alist, "(");
+  for (i = 0; i < num_fields; i++)
+    {
+      gsql_field *field;
+      char *value;
+
+      field = gsql_fetch_field (query_result, i);
+
+      fieldname = gsql_field_name (field);
+
+      value = gsql_get_column (query_result, i);
+      bprintf (alist, "(\"%s\" . \"%s\")", fieldname, value);
+      xfree (value);
+    }
+
+  bprintf (alist, ")");
+  val = strdup (alist->buffer);
+  bprintf_free_buffer (alist);
+  return val;
+
+}
+
+/* List all the table information for a database. This must be done
+   with a database already open, i.e., within the scope of a
+   <msql::with-open-database>.
+
+   Args: <msql::database-tables-info dbvar [result=VAR]>
+   Returns an array of alists, one per table, 
+   or sets the value of VAR if supplied. 
+
+   Args: <odbc::database-tables-info dbvar [result=VAR] 
+   [tabletype=type] [tablequalifier=qualifier] [tablename=name] 
+   [tableowner=owner]>
+
+   The optional tablemumble args are ANSI SQL regexp patterns.
+
+   Each alist contains database-specific information, including at least
+   the NAME of the table.
+   ((name . "table_name"))
+
+*/
+
+static void
+pf_database_tables_info (PFunArgs)
+{
+  char *qualifier = mhtml_evaluate_string (get_value (vars, "tablequalifier"));
+  char *owner     = mhtml_evaluate_string (get_value (vars, "tableowner"));
+  char *name      = mhtml_evaluate_string (get_value (vars, "tablename"));
+  char *type      = mhtml_evaluate_string (get_value (vars, "tabletype"));
+
+  /* No errors yet! */
+  gsql_clear_error_message ();
+
+  if (database_environment_level != 0)
+    {
+      Database *db = get_dbref (vars);
+
+      if ((db != (Database *)NULL) && gsql_database_connected (db))
+	{
+	  /* This returns a result set, with one row of info 
+	     for each table in the database. */
+	  gsql_result *result = gsql_db_list_tables
+	                            (db, qualifier, owner,name, type);
+
+	  if (result != (gsql_result *)NULL)
+	    {
+	      register int i;
+	      char *varname;
+	      char **alists;
+	      int rows;
+	      int arraysize = 10;
+
+	      alists = (char **)
+		xmalloc ((1 + arraysize) * sizeof (char *));
+
+	      /* The name of the variable to stuff the results into. */
+	      varname = mhtml_evaluate_string (get_value (vars, "RESULT"));
+
+	      /* Loop, fetching each row of the result set which
+		 was returned by gsql_db_list_tables. */
+	      i = 0;
+	      while ((gsql_fetch_row (result)) == GSQL_SUCCESS)
+		{
+		  if (i + 2 > arraysize)
+		    alists = (char **)xrealloc
+		      (alists, ((arraysize += 5) * sizeof (char *)));
+
+		  /* Create an alist from this result row.*/
+		  alists[i++] = alist_from_row (result);
+		}
+
+	      alists[i] = (char *) NULL;
+	      rows = i;
+
+	      if (!empty_string_p (varname))
+		{
+		  symbol_store_array (varname, alists);
+		}
+	      else
+		{
+		  for (i = 0; i < rows; i++)
+		    {
+		      bprintf_insert (page, start, "%s\n", alists[i]);
+		      start += 1 + strlen (alists[i]);
+		      free (alists[i]);
+		    }
+		  free (alists);
+
+		  *newstart = start;
+		}
+
+	      xfree (varname);
+	    }
+	}
+    }
+}
