@@ -39,14 +39,39 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
-#include <sys/file.h>
-#include <sys/fcntl.h>
-#if defined (sgi)
+#include <errno.h>
+#if defined (HAVE_SYS_FILE_H)
+#  include <sys/file.h>
+#endif
+
+#if defined (HAVE_FCNTL_H)
 #  include <fcntl.h>
+#else
+#  if defined (HAVE_SYS_FCNTL_H)
+#    include <sys/fcntl.h>
+#  endif
+#endif
+#if defined (HAVE_BSTRING_H)
+#  include <bstring.h>
+#endif
+#include <sys/ioctl.h>
+#if defined (Solaris) && defined (HAVE_SYS_TTOLD_H)
+#  include <sys/ttold.h>
+#endif
+#if defined (TIME_WITH_SYS_TIME)
+#  include <sys/time.h>
+#  include <time.h>
+#else
+#  if defined (HAVE_SYS_TIME_H)
+#    include <sys/time.h>
+#  else
+#    if defined (HAVE_TIME_H)
+#      include <time.h>
+#    endif
+#  endif
 #endif
 #include <sys/stat.h>
-#include <time.h>
-#include <regex.h>
+#include <regex/regex.h>
 
 #include <xmalloc/xmalloc.h>
 #include <bprintf/bprintf.h>
@@ -63,11 +88,27 @@
 #define tag_ender(c)	  (whitespace (c) || (c == '>'))
 
 
-#if !defined (os_open)
-#  define os_open(file, mode, flags) open (file, mode, flags)
-#if defined (Solaris)
-extern int open (char *, int, ...);
+#if defined (macintosh)
+extern char *strdup (const char *string);
+#  define os_open(name, flags, mode) open (name, flags)
 #endif
+
+#if defined (__WINNT__)
+#  if !defined (O_BINARY)
+#    define O_BINARY 0
+#  endif
+#  define os_open(name, flags, mode) open (name, flags | O_BINARY, mode)
+#endif
+#if !defined (os_open)
+#  define os_open(name, flags, mode) open (name, flags, mode)
+#endif
+
+#if !defined (errno)
+extern int errno;
+#endif
+
+#if !defined (xfree)
+#  define xfree(x) if (x) free (x)
 #endif
 
 /* Make PAGE contain CONTENTS, and nothing else. */
@@ -116,11 +157,23 @@ page_read_template (char *passed_filename)
       (strchr (passed_filename, '\\') != (char *)NULL))
     {
       register int i;
+      filename = (char *)xmalloc (4 + strlen (passed_filename));
 
-      filename = strdup (passed_filename);
+      if (passed_filename[1] == ':')
+	{
+	  filename[0] = '/';
+	  filename[1] = '/';
+	  filename[2] = passed_filename[0];
+	  filename[3] = '\0';
+	  i = 2;
+	}
+      else
+	{
+	  i = 0;
+	  filename[0] = 0;
+	}
 
-      if (filename[1] == ':')
-	memmove (filename, filename + 2, 1 + strlen (filename + 2));
+      strcat (filename, passed_filename + i);
 
       for (i = 0; filename[i] != '\0'; i++)
 	if (filename[i] == '\\')
@@ -151,22 +204,13 @@ page_read_template (char *passed_filename)
     {
       int fd;
 
-      fd = open (filename, O_RDONLY, 0666);
+      fd = os_open (filename, O_RDONLY, 0666);
       if (fd > -1)
 	{
 	  int filesize = (int) finfo.st_size;
 	  char *contents = (char *)xmalloc (1 + filesize);
 
-#if defined (__WINNT__)
-	  /* This is so ridiculous.  Can't I just use the information
-	     returned by stat () under Windows NT/95?  No, because the
-	     amount returned by read () is different than the amount
-	     that finfo.st_size reports, based upon CR/LF translation.
-	     Why not just give up entirely? */
-	  if (read (fd, contents, filesize) < 1)
-#else /* If using a normal OS. */
 	  if (read (fd, contents, filesize) != filesize)
-#endif
 	    {
 	      free (contents);
 	    }
@@ -612,8 +656,7 @@ http_date_format (long ticks)
   static char result[100];
 
   static char *weekdays[] = {
-    "Sunday", "Monday", "Tuesday", "Wednesday",
-    "Thursday", "Friday", "Saturday"
+    "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
   };
 
   static char *months[] = {
@@ -621,9 +664,9 @@ http_date_format (long ticks)
     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
   };
 
-  sprintf (result, "%s, %02d-%s-%02d %02d:%02d:%02d GMT",
+  sprintf (result, "%s, %02d %s %04d %02d:%02d:%02d GMT",
 	   weekdays[date->tm_wday],
-	   date->tm_mday, months[date->tm_mon], date->tm_year,
+	   date->tm_mday, months[date->tm_mon], date->tm_year + 1900,
 	   date->tm_hour, date->tm_min, date->tm_sec);
 
   return (result);
