@@ -28,66 +28,15 @@
    Santa Barbara, CA
    93101  */
 
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
 #define COMPILING_PARSER_C 1
+#include "language.h"
 
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <regex.h>
-#include <setjmp.h>
-#include <sys/types.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#endif /* !Solaris */
-#include <time.h>
-#include <math.h>
-
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include <wisper/wisp.h>
-#include "database.h"
-#include "session_data.h"
-#include "pages.h"
-#include "parser.h"
-
-#if defined (MHTML_STREAMS)
-#  include "streamfuncs.h"
-#endif /* MHTML_STREAMS */
-
-#include <locking.h>
-
-#if !defined (HAVE_SRANDOM)
-#  define srandom(seed) srand (seed)
-#  define random() rand()
-#endif
-
-#if defined (HAVE_GETPWNAM)
-#  include <pwd.h>
-#endif /* HAVE_GETPWNAM */
 
 extern void initialize_external_functions (Package *p);
 
-#if defined (macintosh)
-extern char *strdup (const char *string);
-#  define os_open(name, flags, mode) open (name, flags)
-#else
-#  define os_open(name, flags, mode) open (name, flags, mode)
-#endif
-
-
-char *metahtml_copyright_string = "
+char *metahtml_copyright_string = "\
 Copyright (C) 1995, 1996, Brian J. Fox\n\
-Copyright (C) 1996, Universal Access Inc.";
+Copyright (C) 1996, 1997 Universal Access Inc.";
 
 /* Globally known variable holds onto to the reserved words. */
 Package *mhtml_function_package = (Package *)NULL;
@@ -144,6 +93,9 @@ int mhtml_inhibit_comment_parsing = 0;
 int mhtml_decimal_notify = 0;
 int mhtml_decimal_places = 2;
 
+/* Gets 1 when mhtml::parser-trace-mode has a value, 0 otherwise. */
+int mhtml_parser_trace_mode = 0;
+
 void
 pagefunc_initialize_notifiers (void)
 {
@@ -153,6 +105,8 @@ pagefunc_initialize_notifiers (void)
   symbol_notify_value (sym, &mhtml_inhibit_comment_parsing);
   sym = symbol_intern ("mhtml::decimal-places");
   symbol_notify_value (sym, &mhtml_decimal_notify);
+  sym = symbol_intern ("mhtml::parser-trace-mode");
+  symbol_notify_value (sym, &mhtml_parser_trace_mode);
 }
 
 static int syntax_checking = 0;
@@ -790,6 +744,33 @@ package_to_alist (Package *package, int strip)
   return (result);
 }
 
+static int parser_trace_indentation_counter = 0;
+
+static char *
+parser_trace_indentation (void)
+{
+  static char *blanks = (char *)NULL;
+  static int blanks_size = 0;
+  int indent_amount = 2 * parser_trace_indentation_counter;
+
+  if (!indent_amount)
+    return ("");
+
+  if ((indent_amount + 2) > blanks_size)
+    {
+      register int i;
+
+      blanks = (char *)xrealloc
+	(blanks, (blanks_size += (indent_amount + 50)));
+      for (i = 0; i < blanks_size; i++)
+	blanks[i] = ' ';
+      blanks[i] = '\0';
+    }
+
+  /* Return the end of the string. */
+  return (blanks + (blanks_size - indent_amount));
+}
+
 /* Actually process PAGE in place.  The result of processing PAGE is placed
    within PAGE.  This is likely to change shortly, when we pre-parse the
    PAGE and write sequential output to a different destination. */
@@ -879,7 +860,15 @@ page_process_page_internal (PAGE *page)
 	    {
 	      desc = &uf_desc;
 	      desc->tag = uf->name;
-	      desc->complexp = (uf->type == user_MACRO);
+	      if (uf->type == user_MACRO)
+		{
+		  if (uf->flags & user_WEAK_MACRO)
+		    desc->complexp = -1;
+		  else
+		    desc->complexp = 1;
+		}
+	      else
+		desc->complexp = 0;
 	      desc->debug_level = uf->debug_level;
 	      desc->fun = (PFunHandler *)NULL;
 	    }
@@ -897,12 +886,26 @@ page_process_page_internal (PAGE *page)
 	  else
 	    {
 	      int start, end;
-	      int found;
+	      int found, complexp = desc->complexp;
 
 	      start = search_start;
 
 	      if (desc->complexp)
-		found = page_complex_tag_bounds (page, fname, &start, &end);
+		{
+		  int saved = start;
+		  found = page_complex_tag_bounds (page, fname, &start, &end);
+
+		  /* If a closer wasn't found, and this is a weak macro,
+		     then treat this occurence as if the tag didn't
+		     require a closer. */
+		  if (!found && (desc->complexp == -1))
+		    {
+		      start = saved;
+		      end = page_find_tag_end (page, start);
+		      found = end != -1;
+		      complexp = 0;
+		    }
+		}
 	      else
 		{
 		  end = page_find_tag_end (page, start);
@@ -912,7 +915,7 @@ page_process_page_internal (PAGE *page)
 	      if (!found)
 		{
 		  /* The MTHML programmer didn't close the opener correctly.
-		     Ignore the opener, and move on. */
+		     Tell them, and then ignore the text. */
 		  page_debug ("Closing tag missing for <%s ...>", desc->tag);
 		  if (syntax_checking)
 		    {
@@ -932,7 +935,7 @@ page_process_page_internal (PAGE *page)
 		  /* For simple and complex tags alike, we want to eat the
 		     variables which appear in the opener. */
 		  open_start = start;
-		  if (desc->complexp)
+		  if (complexp)
 		    open_end = page_find_tag_end (page, start);
 		  else
 		    open_end = end;
@@ -949,8 +952,13 @@ page_process_page_internal (PAGE *page)
 
 		  vars = pagefunc_snarf_vars (open_body, uf ? 0 : 1);
 
-		  if (!desc->complexp)
-		    strbody = open_body;
+		  if (!complexp)
+		    {
+		      if (!desc->complexp)
+			strbody = open_body;
+		      else
+			strbody = strdup ("");
+		    }
 		  else
 		    {
 		      int open_len = open_end - open_start;
@@ -969,7 +977,7 @@ page_process_page_internal (PAGE *page)
 #endif
 
 		      strbody = page_complex_tag_extract
-			(page, desc->tag, &open_start, &end);
+			(page, fname, &open_start, &end);
 
 		      /* Get rid of the opening tag. */
 		      {
@@ -1019,16 +1027,25 @@ page_process_page_internal (PAGE *page)
 #endif
 			bprintf_delete_range (page, start, end);
 
-		      if (desc->debug_level > 5)
+		      if (mhtml_parser_trace_mode || desc->debug_level > 5)
 			{
 			  display_body = strdup (open_body ? open_body : "");
 			  if (strlen (display_body) > 33)
 			    strcpy (display_body + 30, "...");
 
-			  page_debug ("Entering <%s %s>",
-				      desc->tag, display_body);
+			  if (mhtml_parser_trace_mode)
+			    page_debug ("%sEntering %s: <%s %s>",
+					parser_trace_indentation (),
+					desc->tag, desc->tag,
+					mhtml_funargs (vars));
+			  else
+			    page_debug ("Entering <%s %s>",
+					desc->tag, display_body);
+
 			}
-		    
+
+		      parser_trace_indentation_counter++;
+
 		      if (uf)
 			mhtml_execute_function
 			  (uf, page, body, vars, start, end, &search_start,
@@ -1045,10 +1062,18 @@ page_process_page_internal (PAGE *page)
 			  search_start = page->bindex;
 			}
 
-		      if (desc->debug_level > 5)
+		      parser_trace_indentation_counter--;
+
+		      if (mhtml_parser_trace_mode || desc->debug_level > 5)
 			{
-			  page_debug ("Leaving <%s %s>",
-				      desc->tag, display_body);
+			  if (mhtml_parser_trace_mode)
+			    page_debug ("%sLeaving %s: <%s %s>",
+					parser_trace_indentation (),
+					desc->tag, desc->tag,
+					mhtml_funargs (vars));
+			  else
+			    page_debug ("Leaving <%s %s>",
+					desc->tag, display_body);
 			  free (display_body);
 			}
 
@@ -1206,6 +1231,25 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
 
   uf->flags = 0;
 
+  /* Strip comments now, rather than later. */
+  {
+    register int i = 0;
+    char *b = uf->body;
+
+    while (b[i] != '\0')
+      {
+	if ((b[i] == ';') && (b[i + 1] == ';') && (b[i + 2] == ';'))
+	  {
+	    register int j;
+	    
+	    for (j = i; (b[j] != '\0') && (b[j] != '\n'); j++);
+	    memmove (b + i, b + j, strlen (b + j) + 1);
+	  }
+	else
+	  i++;
+      }
+  }
+     
   /* If the user wants special behaviour for the whitespace present in
      the macro body, then handle it now. */
   if (body_whitespace != (char *)NULL)
@@ -1698,9 +1742,15 @@ mhtml_funargs (Package *pack)
 	    bprintf (string, " ");
 
 	  if ((values[i] != (char *)NULL) && (values[i][0] != '\0'))
-	    bprintf (string, "%s=%s", names[i], values[i]);
+	    {
+	      /* Place quotes around this string on the way out. */
+	      bprintf (string, "%s=\"%s\"", names[i], values[i]);
+	    }
 	  else
-	    bprintf (string, "%s", names[i]);
+	    {
+	      /* Just print the attribute name. */
+	      bprintf (string, "%s", names[i]);
+	    }
 	}
 
       result = string->buffer;
