@@ -28,44 +28,9 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-extern int open (char *, int, ...);
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <fcntl.h>
-#  endif
-#endif /* !Solaris */
-#include <signal.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "forms.h"
-#include "session.h"
-#include "pages.h"
-#include "parser.h"
+#include "language.h"
 #include "symdump.h"
 
-#if defined (HAVE_GETPWNAM)
-#  include <pwd.h>
-#endif
-
 static void pf_eval (PFunArgs);
 static void pf_read (PFunArgs);
 static void pf_write_package_file (PFunArgs);
@@ -146,7 +111,7 @@ pf_write_package_file (PFunArgs)
 
   if (!empty_string_p (filename))
     {
-      int fd = open (filename, O_WRONLY | O_TRUNC | O_CREAT, 0666);
+      int fd = os_open (filename, O_WRONLY | O_TRUNC | O_CREAT, 0666);
 
       if (fd > -1)
 	{
@@ -209,7 +174,7 @@ pf_read_package_file (PFunArgs)
 	    }
 	}
 
-      fd = open (filename, O_RDONLY, 0666);
+      fd = os_open (filename, O_RDONLY, 0666);
 
       if (fd > -1)
 	{
