@@ -28,40 +28,7 @@
    Santa Barbara, CA
    93101  */
 
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <math.h>
-#  endif
-#endif /* !Solaris */
-
-#include <locking.h>
-#if !defined (HAVE_SRANDOM)
-#  include <math.h>
-#  define srandom(seed) srand (seed)
-#  define random() rand()
-#endif
-
-#include <time.h>
+#include "mhtmlstd.h"
 #include "session.h"
 
 #define SESSION_READ  1
@@ -498,7 +465,7 @@ session_lock_database (int *lock, DBFILE *db, int mode)
   else
     dbmode = DB_WRCREAT;
 
-  fd = open (SESSION_LOCKFILE, O_CREAT | O_WRONLY | O_APPEND, 0666);
+  fd = os_open (SESSION_LOCKFILE, O_CREAT | O_WRONLY | O_APPEND, 0666);
 
   if ((fd == -1) ||
       (((mode == SESSION_READ) && (READLOCKFILE (fd) == -1)) ||
