@@ -28,53 +28,8 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <fcntl.h>
-#  endif
-#endif /* !Solaris */
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <signal.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
+#include "language.h"
 #include "session_data.h"
-#include "pages.h"
-#include "parser.h"
-
-#include <locking.h>
-
-#if defined (Solaris)
-extern int open (const char *, int, ...);
-#endif
-
-#if defined (macintosh)
-extern char *strdup (const char *string);
-#  define os_open(name, flags, mode) open (name, flags)
-#else
-#  define os_open(name, flags, mode) open (name, flags, mode)
-#endif
-
-extern jmp_buf page_jmp_buffer;
 
 /************************************************************/
 /*							    */
@@ -239,7 +194,7 @@ pf_create_session (PFunArgs)
 	}
     }
 
-  session_reap ();
+  /* session_reap (); */
   if (key_buffer)
     {
       sid = session_begin (key_buffer->buffer, allow_multiple_p);
