@@ -28,49 +28,7 @@
    Santa Barbara, CA
    93101  */
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <fcntl.h>
-#  endif
-#endif /* !Solaris */
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <signal.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include <errno.h>
-#include "forms.h"
-#include "session.h"
-#include "pages.h"
-#include "parser.h"
-#include "streamfuncs.h"
-
-#if !defined (errno)
-extern int errno;
-#endif
-
-#if defined (Solaris)
-extern int open (char *, int, ...);
-#endif
+#include "language.h"
 
 static void pf_with_open_stream (PFunArgs);
 static void pf_stream_put (PFunArgs);
@@ -89,14 +47,14 @@ PFunDesc streamfunc_table[] = {
   { "STREAM-GET-CONTENTS",	0, 0, pf_stream_get_contents },
   { "STREAM-READABLE",		0, 0, pf_stream_readable },
   { "STREAM-WRITABLE",		0, 0, pf_stream_writable },
-  { "STREAM-WRITEABLE",		0, 0, pf_stream_writable },
   { "STREAM-SHUTDOWN",		0, 0, pf_stream_shutdown },
   { (char *)NULL,		0, 0, (PFunHandler *)NULL }
 };
 
 typedef enum { stream_FILE, stream_TCP, stream_CLOSED } StreamType;
 
-#define stream_MUST_SEEK 1
+#define stream_MUST_SEEK 0x01
+#define stream_NOTIMEOUT 0x02
 #define op_NONE  0
 #define op_READ  1
 #define op_WRITE 2
@@ -242,13 +200,15 @@ pf_with_open_stream (PFunArgs)
     {
       char *modename = mhtml_evaluate_string (get_value (vars, "MODE"));
       char *typename = mhtml_evaluate_string (get_value (vars, "TYPE"));
+      char *notimeout = mhtml_evaluate_string (get_value (vars, "NOTIMEOUT"));
       int mode = stream_mode (modename);
       int type = stream_type (typename);
-      int flags = 0;
+      int flags = notimeout ? stream_NOTIMEOUT : 0;
       int stream_fd = -1;
 
       if (modename) free (modename);
       if (typename) free (typename);
+      if (notimeout) free (notimeout);
 
       switch (type)
 	{
@@ -260,7 +220,7 @@ pf_with_open_stream (PFunArgs)
 	  else if (strcasecmp (filename, "*standard-error*") == 0)
 	    stream_fd = mhtml_stderr_fileno;
 	  else
-	    stream_fd = open (filename, mode, 0666);
+	    stream_fd = os_open (filename, mode, 0666);
 	  break;
 
 	case stream_TCP:
@@ -459,9 +419,10 @@ stream_read (Stream *stream, char char_to_stop_at)
   if ((stream->flags & stream_MUST_SEEK) && (stream->last_op != op_READ))
     lseek (stream->fd, 0l, 1);
 
-  operation_timed_out_p = 0;
-  signal (SIGALRM, operation_timed_out);
-  alarm (20);
+  if (stream->flags & stream_NOTIMEOUT)
+    set_io_timer (999999);
+  else
+    set_io_timer (80);
 
   if (setjmp (operation_jmp_buf))
     {
@@ -582,7 +543,10 @@ stream_read_chunk (Stream *stream, int length)
 
   buffer = (char *)xmalloc (length + 1);
 
-  set_io_timer (length);
+  if (stream->flags & stream_NOTIMEOUT)
+    set_io_timer (999999);
+  else
+    set_io_timer (length);
 
   if (setjmp (operation_jmp_buf))
     {
