@@ -776,6 +776,31 @@ symbol_free (Symbol *symbol)
     }
 }
 
+void
+symbol_reset (Symbol *symbol)
+{
+  switch (symbol->type)
+    {
+    case symtype_STRING:
+      free_array (symbol->values);
+      break;
+
+    case symtype_BINARY:
+      datablock_free ((Datablock *)symbol->values);
+      break;
+
+    case symtype_FUNCTION:
+    case symtype_USERFUN:
+      return;
+    }
+
+  symbol->flags = 0;
+  symbol->type = symtype_STRING;
+  symbol->values_index = 0;
+  symbol->values_slots = 0;
+  symbol->values = (char **)NULL;
+}
+
 int
 symbol_get_flags (Symbol *symbol)
 {
