@@ -50,16 +50,166 @@
 #  include <mac_port.h>
 #endif
 
+/* Functions for reading from and writing to a string. */
+#define STRING_FD_THRESHOLD 1024
+#define FD_IS_STRING_BUFFER(fd) ((fd) >= STRING_FD_THRESHOLD)
+#define SD_TO_BUFFER_INDEX(sd) ((sd) - STRING_FD_THRESHOLD)
+#define BUFFER_INDEX_TO_SD(bi) ((bi) + STRING_FD_THRESHOLD)
+
+static BPRINTF_BUFFER **open_strings = (BPRINTF_BUFFER **)NULL;
+static int open_strings_slots = 0;
+static int open_strings_index = 0;
+
+static BPRINTF_BUFFER *
+buffer_of_sd (int sd)
+{
+  BPRINTF_BUFFER *result = (BPRINTF_BUFFER *)NULL;
+
+  sd = SD_TO_BUFFER_INDEX (sd);
+
+  if ((sd >= 0) && (sd < open_strings_index))
+    result = open_strings[sd];
+
+  return (result);
+}
+
+/* Read LEN bytes from SD into DATA. */
+int
+symdump_read_string_data (int sd, int len, unsigned char *data)
+{
+  BPRINTF_BUFFER *string_buffer = buffer_of_sd (sd);
+  int result = 0;
+
+  if (string_buffer != (BPRINTF_BUFFER *)NULL)
+    {
+      if (string_buffer->bindex + len <= string_buffer->bsize)
+	{
+	  register int i;
+
+	  for (i = 0; i < len; i++)
+	    data[i] = (unsigned char)
+	      (string_buffer->buffer[string_buffer->bindex++]);
+	  result = i;
+	}
+    }
+  return (result);
+}
+
+/* Write LEN bytes to SD from DATA. */
+int
+symdump_write_string_data (int sd, int len, unsigned char *data)
+{
+  BPRINTF_BUFFER *string_buffer = buffer_of_sd (sd);
+  int result = 0;
+
+  if (string_buffer != (BPRINTF_BUFFER *)NULL)
+    {
+      register int i;
+
+      if (len + string_buffer->bindex >= string_buffer->bsize)
+	string_buffer->buffer = (char *)xrealloc
+	  (string_buffer->buffer, (string_buffer->bsize += (len + 100)));
+
+      for (i = 0; i < len; i++)
+	((unsigned char *) (string_buffer->buffer))[string_buffer->bindex++] =
+	  data[i];
+
+      result = i;
+    }
+
+  return (result);
+}
+
+/* Set the current offstring in SD to OFFSET. */
+void
+symdump_seek_string_data (int sd, int offset)
+{
+  BPRINTF_BUFFER *string_buffer = buffer_of_sd (sd);
+
+  if ((string_buffer != (BPRINTF_BUFFER *)NULL) &&
+      ((offset >= 0) && (offset <= string_buffer->bsize)))
+    string_buffer->bindex = offset;
+}
+
+/* Set the current offstring in SD to OFFSET. */
+void
+symdump_set_string_data_buffer_size (int sd, int end)
+{
+  BPRINTF_BUFFER *string_buffer = buffer_of_sd (sd);
+
+  if ((string_buffer != (BPRINTF_BUFFER *)NULL) &&
+      ((end >= 0) && (end <= string_buffer->bsize)))
+    string_buffer->bsize = end;
+}
+
+/* Return an SD (like an FD, but for the string buffer pool.
+   Newly allocates a buffer.
+   The SD returned is suitable for passing to symbol_load_package ()
+   and symbol_dump_package (). */
+int
+symdump_open_string_data (void)
+{
+  register int i;
+  int sd = -1;
+
+  if (open_strings_index + 2 > open_strings_slots)
+    {
+      open_strings = (BPRINTF_BUFFER **) xrealloc
+	(open_strings, (open_strings_slots += 4) * sizeof (BPRINTF_BUFFER *));
+      for (i = open_strings_index; i < open_strings_slots; i++)
+	open_strings[i] = (BPRINTF_BUFFER *)NULL;
+    }
+
+  /* Find first unused slot. */
+  for (i = 0; i < open_strings_slots; i++)
+    {
+      if (open_strings[i] == (BPRINTF_BUFFER *)NULL)
+	{
+	  sd = i + STRING_FD_THRESHOLD;
+	  open_strings[i] = bprintf_create_buffer ();
+	  if (i == open_strings_index)
+	    open_strings_index++;
+	  break;
+	}
+    }
+
+  return (sd);
+}
+
+/* Return the BPRINTF_BUFFER corresponding to SD, and return the allocated
+   slot back to the pool.  Does NOT free the BPRINTF_BUFFER. */
+BPRINTF_BUFFER *
+symdump_close_string_data (int sd)
+{
+  BPRINTF_BUFFER *string_buffer = buffer_of_sd (sd);
+
+  if (string_buffer != (BPRINTF_BUFFER *)NULL)
+    {
+      /* Reset the end of the buffer. */
+      string_buffer->bsize = string_buffer->bindex;
+
+      /* Allow slot to be reused. */
+      open_strings[SD_TO_BUFFER_INDEX (sd)] = (BPRINTF_BUFFER *)NULL;
+    }
+  return (string_buffer);
+}
+
 static int
 write_data (int fd, int len, unsigned char *data)
 {
-  return (write (fd, data, len));
+  if (FD_IS_STRING_BUFFER (fd))
+    return (symdump_write_string_data (fd, len, data));
+  else
+    return (write (fd, data, len));
 }
 
 static int
 read_data (int fd, int len, unsigned char *data)
 {
-  return (read (fd, data, len));
+  if (FD_IS_STRING_BUFFER (fd))
+    return (symdump_read_string_data (fd, len, data));
+  else
+    return (read (fd, data, len));
 }
 
 static int
@@ -87,15 +237,19 @@ static int
 read_integer (int fd)
 {
   unsigned char b1, b2, b3, b4;
+  int safety = 0;
   int integer = 0;
 
-  read_data (fd, 1, &b1);
-  read_data (fd, 1, &b2);
-  read_data (fd, 1, &b3);
-  read_data (fd, 1, &b4);
+  safety += read_data (fd, 1, &b1);
+  safety += read_data (fd, 1, &b2);
+  safety += read_data (fd, 1, &b3);
+  safety += read_data (fd, 1, &b4);
 
-  integer =
-    (((int)b4) << 24) | (((int)b3) << 16) | (((int)b2) << 8) | ((int)b1);
+  if (safety != 4)
+    integer = 0;
+  else
+    integer =
+      (((int)b4) << 24) | (((int)b3) << 16) | (((int)b2) << 8) | ((int)b1);
 
   return (integer);
 }
@@ -130,8 +284,17 @@ write_symbol (Symbol *symbol, int fd)
       /* Write the values themselves. */
       for (i = 0; i < symbol->values_index; i++)
 	{
-	  write_integer (fd, strlen (symbol->values[i]));
-	  write_data (fd, strlen (symbol->values[i]), symbol->values[i]);
+	  int len = symbol->values[i] ? strlen (symbol->values[i]) : 0;
+	  char *data_pointer = symbol->values[i];
+
+	  if (!len)
+	    {
+	      len = 1;
+	      data_pointer = "";
+	    }
+
+	  write_integer (fd, len);
+	  write_data (fd, len, symbol->values[i]);
 	}
       break;
 
@@ -377,7 +540,7 @@ symbol_load_package (int fd)
   Package *package = (Package *)NULL;
 
   /* Read the length of the package name. */
-  if (read_data (fd, 1, &byte) == 0)
+  if (read_data (fd, 1, &byte) < 1)
     return (package);
 
   if (byte + 1 > buflen)
