@@ -401,10 +401,15 @@ mhttpd_page_redirect_p (PAGE *page)
 
   if ((page != (PAGE *)NULL) && (page->buffer != (char *)NULL))
     {
-      char *temp = strstr (page->buffer, "Location: ");
+      register int i;
+      register char *temp;
 
-      if ((temp != (char *)NULL) && ((temp - page->buffer) < 25))
-	result = 1;
+      for (i = 0, temp = page-> buffer; i < 25; i ++, temp ++)
+	if (*temp == 'L' && strncasecmp (temp + 1, "ocation: ", 9) == 0)
+	  {
+	    result = 1;
+	    break;
+	  }
     }
 
   return (result);
@@ -547,7 +552,7 @@ mhttpd_write (int fd, const void *buf, size_t nbytes)
 #endif
     {
       int select_result = 0;
-#if defined (FD_SET)
+#if defined (FD_SET) && !defined (__WINNT__)
       struct timeval timeout = { 0, 0 };
       fd_set except_fds;
       int intr = 0;
@@ -615,7 +620,7 @@ mhttpd_gets (char *str, size_t size, int fd)
       for (i = 0; i < limit; i++)
 	{
 	  int select_result = 1;
-#if defined (FD_SET)
+#if defined (FD_SET) && !defined (__WINNT__)
 	  struct timeval timeout = { 60, 0 };
 	  fd_set read_fds;
 
