@@ -28,8 +28,6 @@
    Santa Barbara, CA
    93101  */
 
-#define COMPILING_HTTP_C 1
-
 #if defined (HAVE_CONFIG_H)
 #  include <config.h>
 #endif
@@ -78,12 +76,38 @@ extern int errno;
 
 extern char **environ;
 
+/* Yechh!  This is solely for making the Open Market Fast CGI implementation
+   work without having to recompile this library separately for the engine.
+   This makes me feel a little sick. */
+char *fast_cgi_content = (char *)NULL;
+int fast_cgi_content_length = 0;
+
 static void mhttpd_fill_in_missing_headers (HTTP_RESULT *result);
 static PAGE *mhttpd_find_page_for_result (HTTP_RESULT *result);
 static void mhttpd_handle_empty_page (HTTP_RESULT *result);
 
 #define word_separator(x) whitespace(x)
 
+static HTTP_RESULT *http_get_handler (ReqFunArgs);
+static HTTP_RESULT *http_head_handler (ReqFunArgs);
+static HTTP_RESULT *http_post_handler (ReqFunArgs);
+static HTTP_RESULT *http_put_handler (ReqFunArgs);
+static HTTP_RESULT *http_delete_handler (ReqFunArgs);
+static HTTP_RESULT *http_link_handler (ReqFunArgs);
+static HTTP_RESULT *http_unlink_handler (ReqFunArgs);
+static HTTP_RESULT *http_unimplemented_handler (ReqFunArgs);
+
+static METHOD_HANDLER method_handlers[] = {
+  { "GET",	http_get_handler },
+  { "HEAD",	http_head_handler },
+  { "POST",	http_post_handler },
+  { "PUT",	http_put_handler },
+  { "DELETE",	http_delete_handler },
+  { "LINK",	http_link_handler },
+  { "UNLINK",	http_unlink_handler },
+  { (char *)NULL, (ReqFun *)NULL }
+};
+
 void
 http_log_error (char *format, ...)
 {
@@ -97,7 +121,7 @@ http_readline (int fd)
   char *line;
 
   if (buffer == (char *)NULL)
-    buffer = (char *)xmalloc (buffer_size = 1024);
+    buffer = (char *)xmalloc (buffer_size = 2048);
 
   line = mhttpd_gets (buffer, buffer_size, fd);
 
@@ -620,7 +644,10 @@ mr_mime_version (HTTP_RESULT *result, char *ignore)
 static char *
 mr_server (HTTP_RESULT *result, char *ignore)
 {
-  return (sv_VersionString);
+  if (!ignore)
+    return (sv_VersionString);
+  else
+    return ((char *)NULL);
 }
 
 static char *
@@ -685,7 +712,7 @@ mr_set_cookie (HTTP_RESULT *result, char *ignore)
 	      if (timeout > 0)
 		{
 		  if (permanent_p)
-		    ticks = 946700000;
+		    ticks = 946684799;
 		  else
 		    ticks += timeout;
 		}
@@ -764,7 +791,15 @@ mr_expires (HTTP_RESULT *result, char *ignore)
       answer = mr_date (result, (char *)- (60 * 60 * 24));
     }
   else
-    answer = mr_date (result, (char *)864000); /* Plus 10 days.  */
+    {
+      char *ttlstr = pagefunc_get_variable ("mhtml::cacheable");
+      long offset_seconds = ttlstr ? atol (ttlstr) : 864000; /* 10 days. */
+
+      if (offset_seconds == 0)
+	offset_seconds = 864000;
+
+      answer = mr_date (result, (char *)offset_seconds);
+    }
 
   return (answer);
 }
@@ -809,18 +844,33 @@ mr_location (HTTP_RESULT *result, char *existing_location)
       char *protocol = downcase (result->request->protocol);
       int port = atoi (pagefunc_get_variable ("mhtml::server-port"));
       char *server_name = pagefunc_get_variable ("mhtml::server-name");
-      char *new = (char *)xmalloc (100 + strlen (existing_location));
+      char *extra = pagefunc_get_variable ("rewriter::engine-webpath");
+      char *new;
+
+      if (extra == (char *)NULL) extra = "";
+
+      new = (char *)xmalloc
+	(100 + strlen (extra) + strlen (existing_location));
 
       sprintf (new, "%s://%s", protocol, server_name);
       if (((strcmp (protocol, "http") == 0) && (port != 80)) ||
 	  ((strcmp (protocol, "https") == 0) && (port != 443)))
 	sprintf (new + strlen (new), ":%d", port);
+      strcat (new, extra);
       strcat (new, existing_location);
       existing_location = new;
     }
   return (existing_location);
 }
 
+/* We always say something about the fact that this is the Meta-HTML engine.
+   Otherwise, it isn't directly obvious when connecting to a user's site. */
+static char *
+mr_engine (HTTP_RESULT *result, char *ignore)
+{
+  return (sv_VersionString);
+}
+
 /* If the user has supplied a status, organize things so that this is the
    status returned. */
 static char *
@@ -898,6 +948,7 @@ static MIME_RESOLVER mime_resolvers[] = {
   { "Last-modified",	mr_last_modified, 0 },
   { "Location",		mr_location, 1 },
   { "Content-length",	mr_content_length, 0 },
+  { "Meta-HTML-Engine",	mr_engine, 1 },
   { "Content-type",	mr_content_type, 1 },
   { "Status",		mr_status, 1 },
 
@@ -982,12 +1033,14 @@ mhttpd_fill_in_missing_headers (HTTP_RESULT *result)
       char *protocol = downcase (result->request->protocol);
       int port = atoi (pagefunc_get_variable ("mhtml::server-port"));
       char *server_name = pagefunc_get_variable ("mhtml::server-name");
+      char *rewrite_prefix=pagefunc_get_variable ("rewriter::engine-webpath");
 
       bprintf (response, "Location: %s://%s", protocol, server_name);
       if (((strcmp (protocol, "http") == 0) && (port != 80)) ||
 	  ((strcmp (protocol, "https") == 0) && (port != 443)))
 	bprintf (response, ":%d", port);
-      bprintf (response, "%s\n", result->spec->logical_path);
+      bprintf (response, "%s%s\n", rewrite_prefix ? rewrite_prefix : "",
+	       result->spec->logical_path);
     }
 
   if (mhttpd_debugging)
@@ -996,7 +1049,6 @@ mhttpd_fill_in_missing_headers (HTTP_RESULT *result)
   bprintf_insert (result->page, 0, "%s\n", response->buffer);
   bprintf_free_buffer (response);
 
-
 #if 0
   if (present)
     free_mime_headers (present);
@@ -1167,7 +1219,14 @@ mhtml_read_content_from_fd (HTTP_RESULT *result, int fd)
 
       result->spec->content = (char *)xmalloc
 	(1 + result->spec->content_length);
-      mhttpd_read (fd, result->spec->content, result->spec->content_length);
+
+      /* Handle Fast CGI cruft.  Ugh. */
+      if (fast_cgi_content_length > 0)
+	memmove (result->spec->content, fast_cgi_content,
+		 fast_cgi_content_length);
+      else
+	mhttpd_read (fd, result->spec->content, result->spec->content_length);
+
       result->spec->content[result->spec->content_length] = '\0';
 
 #if defined (GUESS_TRANSFER_RATE)
@@ -2022,6 +2081,51 @@ mhttpd_execute_cgi (HTTP_RESULT *result, int fd)
   return (page);
 }
 
+/* GMT formatted dates in HTTP headers look like one of the following:
+
+   1) Sun, 06 Nov 1994 08:49:37 GMT    ; RFC 822, updated by RFC 1123
+   2) Sunday, 06-Nov-94 08:49:37 GMT   ; RFC 850, obsoleted by RFC 1036 */
+static void
+mhttpd_gmt_canonicalize (char *time_string)
+{
+  register int i;
+  int dashes_found = 0;
+
+  /* Format 1 always has day names that are longer than 3 characters.
+     If this day name is longer, then truncate it. */
+  for (i = 0; (time_string[i] != ','); i++);
+
+  if (i > 3)
+    memmove (time_string + 3, time_string + i, 1 + strlen (time_string + i));
+
+  /* Now remove all `-' characters. */
+  for (i = 0; time_string[i] != '\0'; i++)
+    {
+      if (time_string[i] == '-')
+	{
+	  dashes_found++;
+	  time_string[i] = ' ';
+
+	  if (dashes_found == 2)
+	    {
+	      /* The next two characters depict the year of the document.
+		 If the year is less than 100, add 1900 to it. */
+	      register int j;
+
+	      for (j = i + 1; isdigit (time_string[j]); j++);
+
+	      if ((j - i) < 4)
+		{
+		  memmove (time_string + j, time_string + i + 1,
+			   strlen (time_string + i));
+		  time_string[i + 1] = '1';
+		  time_string[i + 2] = '9';
+		}
+	    }
+	}
+    }
+}
+
 /* Given a complete, basically ready to return document in RESULT, determine
    if we should actually return it or not.  We do this by checking for the
    presence of the If-Modified-Since header in the request, and actually
@@ -2047,6 +2151,15 @@ mhttpd_handle_if_modified_since (HTTP_RESULT *result)
     {
       char *temp;
 
+      /* GMT formatted dates in HTTP headers look like one of the following:
+
+	 1) Sun, 06 Nov 1994 08:49:37 GMT    ; RFC 822, updated by RFC 1123
+	 2) Sunday, 06-Nov-94 08:49:37 GMT   ; RFC 850, obsoleted by RFC 1036
+
+	 So, canonicalize the date of the if-modified header and the
+	 date of the file into the same format, and then do direct string
+	 comparisons between them. */
+
       ims = strdup (ims);
       temp = strchr (ims, ';');
       if (temp != (char *)NULL)
@@ -2056,6 +2169,8 @@ mhttpd_handle_if_modified_since (HTTP_RESULT *result)
 	  while ((temp != ims) && (whitespace (*temp))) --temp;
 	  temp++;
 	  *temp = '\0';
+
+	  mhttpd_gmt_canonicalize (ims);
 	}
     }
 
@@ -2063,6 +2178,11 @@ mhttpd_handle_if_modified_since (HTTP_RESULT *result)
     {
       char *filetime = http_date_format ((long) finfo.st_mtime);
 
+      mhttpd_gmt_canonicalize (filetime);
+
+      mhttpd_log (log_DEBUG, "\nIf Modified Since:\n\t`%s'\n\t`%s'\n\n",
+		  ims, filetime);
+
       if (strcasecmp (ims, filetime) == 0)
 	{
 	  /* The document hasn't been modified since the client last received
@@ -2111,11 +2231,13 @@ http_get_handler (ReqFunArgs)
 	      break;
 
 	    case doc_SERVER_REDIRECT:
-	      result->page = page_create_page ();
-	      bprintf (result->page, "Location: %s\n\n",
-		       result->spec->logical_path);
-	      result->result_code = res_MOVED_TEMPORARILY;
-	      break;
+	      {
+		result->page = page_create_page ();
+		bprintf (result->page, "Location: %s\n\n",
+			 result->spec->logical_path);
+		result->result_code = res_MOVED_TEMPORARILY;
+		break;
+	      }
 	    }
 	}
       else
