@@ -246,7 +246,7 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
 
       if (the_filename[the_filename_len - 1] == '/')
 	{
-	  the_filename[the_filename_len - 1] = '\0';
+	  /* the_filename[the_filename_len - 1] = '\0'; */
 
 	  if ((stat (the_filename, &finfo) != -1) && (S_ISDIR (finfo.st_mode)))
 	    {
@@ -260,8 +260,8 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
 		  for (i = 0; default_filenames[i] != (char *)NULL; i++)
 		    {
 		      stat_path =
-			mhtml_concat (3, the_filename, "/",
-				      default_filenames[i]);
+			mhtml_concat (2, the_filename, default_filenames[i]);
+
 		      if (stat (stat_path, &finfo) != -1)
 			{
 			  use_this_name++;
@@ -486,9 +486,11 @@ mhttpd_resolve_location (HTTP_REQUEST *request)
   /* If there is PATH_INFO, then find it in the filename, and remove it. */
   if (spec->path_info != (char *)NULL)
     {
-      temp = strstr (the_filename, spec->path_info);
-      if (temp != (char *)NULL)
-	*temp = '\0';
+      int l = strlen (spec->path_info);
+      int y = strlen (the_filename) - l;
+
+      if (strcmp (the_filename + y, spec->path_info) == 0)
+	the_filename[y] = '\0';
     }
 
   /* From here on out, the physical path will not change its location.
