@@ -28,27 +28,7 @@
    Santa Barbara, CA
    93101  */
 
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/file.h>
-#include <sys/stat.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "pages.h"
-#include "session_data.h"
-#include "parser.h"
+#include "language.h"
 #include "mdb.h"
 #include "commands.h"
 #include "breakpoints.h"
@@ -991,10 +971,6 @@ mdb_breakpoint (MDBArgs)
   return (result);
 }
       
-static void
-mdb_apropos (MDBArgs)
-{
-}
 
 static int include_recursive_calls = 0;
 
