@@ -27,35 +27,13 @@
    129 El Paseo Court
    Santa Barbara, CA
    93101  */
-
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
+#include "language.h"
+#include "mdb.h"
+#include "commands.h"
 #if !defined (MHTML_SYSTEM_TYPE)
 #  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
 #endif
 
-#define LANGUAGE_DEFINITIONS_FILE 1
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <setjmp.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "pages.h"
-#include "session_data.h"
-#include "parser.h"
-#include "mdb.h"
-#include "commands.h"
-
 extern char *readline (char *prompt);
 extern void add_history (char *string);
 extern void mdb_initialize_readline (void);
@@ -87,13 +65,14 @@ main (int argc, char *argv[])
   if (!*metahtml_copyright_string)
     abort ();
 
-  printf ("MDB 2.0: Interactive Debugger for Meta-HTMLv%s, by Brian J. Fox.",
+  printf ("MDB 2.1: Interactive Debugger for Meta-HTMLv%s, by Brian J. Fox.",
 	  mhtml_version_string);
-  printf ("%s\n", metahtml_copyright_string);
+  printf ("\n%s\n", metahtml_copyright_string);
   printf ("Compiled for (%s)\n", MHTML_SYSTEM_TYPE);
   printf ("CAUTION!  This is experimental software; bugs certainly exist.\n");
 
   initialize_mdb (argc, argv);
+  bootstrap_metahtml (1);
 
   if (setjmp (mdb_top_level_jmp_buffer))
     {
