@@ -28,28 +28,11 @@
    Santa Barbara, CA
    93101  */
 
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
-
+#include "language.h"
 #if !defined (MHTML_SYSTEM_TYPE)
 #  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
 #endif
 
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <signal.h>
-#include <sys/stat.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include "pages.h"
-#include "session_data.h"
-#include "parser.h"
-
 #if !defined (MHTML_VERSION_STRING)
 static char *mhtml_version_string = "";
 #else
@@ -113,7 +96,8 @@ usage (void)
 {
   if (isatty (fileno (stdin)))
     {
-      fprintf (stderr, "mhc: %s\n\n", metahtml_copyright_string);
+      fprintf (stderr, "mhc: %s%s\n\n", MHTML_SYSTEM_TYPE,
+	       metahtml_copyright_string);
       fprintf (stderr, "Usage: %s [--set NAME VALUE]... FILENAME\n", progname);
     }
   exit (1);
@@ -188,6 +172,7 @@ parse_program_args (int argc, char *argv[])
   char *filename = (char *)NULL;
 
   progname = argv[0];
+  pagefunc_set_variable ("mhc::script-name", progname);
 
   while (arg_index < argc)
     {
