@@ -28,59 +28,15 @@
    Santa Barbara, CA
    93101  */
 
-#if defined (HAVE_CONFIG_H)
-#  include <config.h>
-#endif
+#include "language.h"
+#include "globals.h"
+#include "logging.h"
 
+#define I_UNDERSTAND_IDENT
 #if !defined (MHTML_SYSTEM_TYPE)
 #  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
 #endif
 
-#define I_UNDERSTAND_IDENT
-
-#include <stdio.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#if defined (sgi)
-#include <bstring.h>
-#endif
-#include <errno.h>
-#include <sys/types.h>
-#if defined (Solaris)
-#  include <ucbinclude/sys/fcntl.h>
-#  include <ucbinclude/sys/file.h>
-extern int open (char *, int, ...);
-#else
-#  include <sys/file.h>
-#  if defined (sgi)
-#    include <fcntl.h>
-#  endif
-#endif /* !Solaris */
-#include <sys/ioctl.h>
-#if defined (Solaris)
-#  include <sys/ttold.h>
-#endif
-#include <sys/socket.h>
-#include <sys/wait.h>
-#include <sys/time.h>
-#include <sys/stat.h>
-#include <netinet/in.h>
-#include "signal_defs.h"
-#include <netdb.h>
-#include <bprintf/bprintf.h>
-#include <xmalloc/xmalloc.h>
-#include <tcp/tcp.h>
-#include "pages.h"
-#include "symbols.h"
-#include "parser.h"
-#include "session.h"
-#include "streamfuncs.h"
-#include "http.h"
-#include "globals.h"
-#include "logging.h"
-
 #if defined (HAVE_GETPWNAM)
 #  include <pwd.h>
 #endif
@@ -93,8 +49,6 @@ extern int errno;
 extern int killpg (pid_t, int);
 #endif
 
-extern int tcp_to_host (char *host, char *service);
-
 #if !defined (SEEK_END)
 #  define SEEK_END 2
 #endif
@@ -152,6 +106,10 @@ static void fatal (char *format, ...);
 static void check_timestamp (char *filename);
 #endif
 
+#if defined (CHECK_ACTIVATION)
+extern int check_activation_key (char *key);
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -189,7 +147,6 @@ main (int argc, char *argv[])
 	 up the controlling terminal. */
       if (!pid)
 	{
-	  int tty;
 	  char server_pid[20];
 	  char *prefix = (char *)xmalloc (20 + strlen (mhttpd_config_path));
 	  FILE *pid_stream;
@@ -197,7 +154,7 @@ main (int argc, char *argv[])
 
 	  /* Save the pid of this process in the same directory as the
 	     mhttpd.conf file. */
-	  sprintf (server_pid, "%d", getpid ());
+	  sprintf (server_pid, "%d", (int)getpid ());
 	  sprintf (prefix, "%s", mhttpd_config_path);
 
 	  temp = strrchr (prefix, '/');
@@ -244,15 +201,18 @@ main (int argc, char *argv[])
 	    }
 
 #if defined (TIOCNOTTY)
-	  tty = open ("/dev/tty", O_RDWR, 0666);
-	  if (tty) ioctl (tty, TIOCNOTTY, 0);
-	  close (tty);
+	  {
+	    int tty;
+	    tty = open ("/dev/tty", O_RDWR, 0666);
+	    if (tty) ioctl (tty, TIOCNOTTY, 0);
+	    close (tty);
+	  }
 #endif
-#if defined (__linux__) || defined (Solaris) || defined (sgi) || defined (hpux) || defined (__osf__)
+#if defined (SETPGRP_VOID)
 	  setpgrp ();
 #else
 	  setpgrp (0, getpid());
-#endif
+#endif /* !SETPGRP_VOID */
 	  close (0);
 	  close (1);
 
@@ -454,6 +414,21 @@ index.html\nhome.html");
       chdir (sv_DocumentRoot);
     }
 
+  /* Create a reasonable default PATH variable if the user didn't do so. */
+  if (pagefunc_get_variable ("mhtml::exec-path") == (char *)NULL)
+    {
+      page = page_create_page ();
+      bprintf (page, "<set-var mhtml::exec-path =");
+      bprintf (page, "<get-var mhtml::server-root>/bin:/bin:");
+      bprintf (page, "<get-var mhtml::document-root>/cgi-bin:/usr/bin:");
+      bprintf (page, "/usr/local/bin:/usr/ucb:/www/bin:/opt/metahtml/bin");
+      bprintf (page, ">");
+      page_process_page (page);
+
+      if (page)
+	page_free_page (page);
+    }
+
   /* Now to the logfiles. */
   mhttpd_set_logfile (log_ACCESS,  "");
   mhttpd_set_logfile (log_ERROR,   "");
@@ -593,6 +568,16 @@ initialize_server ()
 
   pagefunc_set_variable ("mhtml::system-type", MHTML_SYSTEM_TYPE);
   reinitialize_server ();
+  bootstrap_metahtml (0);
+
+#if defined (CHECK_ACTIVATION)
+  {
+    char *key = pagefunc_get_variable ("mhtml::activation-key");
+    if (check_activation_key (key) == 0)
+      exit (1);
+    symbol_free (symbol_remove ("mhtml::activation-key"));
+  }
+#endif /* CHECK_ACTIVATION */
 
   if (mhttpd_debugging)
     mhttpd_log (log_ERROR, "%s: Started server on port %s",
@@ -874,7 +859,7 @@ check_child_timeouts (void)
 static void
 shutdown_cleanly (int sig)
 {
-#if defined (hpux)
+#if defined (hpux) || defined (__WINNT__)
   kill (-getpgrp(), sig);
 #else
   killpg (0, sig);
